@@ -3,6 +3,7 @@ import typing
 import argparse
 import logging
 from textwrap import dedent
+from fractions import Fraction
 
 from pathlib import Path
 from functools import wraps, singledispatchmethod
@@ -13,13 +14,13 @@ from datetime import datetime
 import random
 
 import tkinter as tk
-from . import LOGGING_VERBOSE
-from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform, ScrolledWidget, FrameRadio, SecondaryWindow
+from . import LOGGING_VERBOSE, AUTO, HORIZONTAL, VERTICAL
+from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform, ScrolledWidget, Scrolled, FrameRadio, SecondaryWindow
 from . import Button, Checkbox, EntryRaw, EntryN, Label, LabelStateful, Listbox, ComboboxN, EntryMultiline, Tree, SpinboxN, Radio, Canvas, SeparatorV, SeparatorH
-from . import ListboxControl, CheckboxFrame, RadioFrame, Tooltip
+from . import ListboxControl, CheckboxFrame, RadioFrameLabelled, RadioFrameStateful, Tooltip
 from . import var, fn, mixin, diagram, bg
-from .model import CP, BindingGlobal, Timeout, TimeoutIdle, Interval, RateLimiter, FileType, FileTypes, Justification, WindowState, GuiState
-from .model import SStyle, DStyle, VSettings, EventModifier
+from .model import CP, BindingGlobal, Timeout, TimeoutIdle, Interval, RateLimiter, FileType, FileTypes, Justification, WindowState, NotebookTabOrder, GuiState
+from .model import SStyle, DStyle, WState, VSettings, EventModifier
 from .validation import LimitBounded, LimitUnbounded, StaticList, StaticMap, StaticMapValues
 try:
     # Try to use nicer pretty-print, from Python 3.10
@@ -77,6 +78,8 @@ class Diagram_SolarSystem(diagram.Diagram):
     C_ISS = diagram.C(fill='grey60', outline='lightblue')
     C_MOON = diagram.C(fill='grey20', outline='grey80')
 
+    # Sun "Shade"
+    SUNSHADE_RANGE = (random.uniform(-80, -15), random.uniform(15, 80))
     # "Pentagon"
     INNER_LAND_EARTH = [random.uniform(4 / 10, 8 / 10) for _ in range(5)]
     INNER_TREE_EARTH = random.sample(list(enumerate(INNER_LAND_EARTH)), k=len(INNER_LAND_EARTH) - 1)
@@ -97,12 +100,23 @@ class Diagram_SolarSystem(diagram.Diagram):
         yield diagram.CircleCenter(center_SUN, radius_SUN,
                                    color=self.C_SUN, colorActive=self.C_SUN.reverse,
                                    width=self.W_SUN,
-                                   tags=['star'])
+                                   tags=['star']), 'sun'
         yield diagram.Text(center_SUN + diagram.VectorH(5), 'Sol',
                            color=diagram.C(fill='black'),
                            anchor=CP.W, font=('TkHeadingFont', 20),
                            tags=['labels'])
 
+        delta_SUNSHADE = radius_SUN // 5
+        yield diagram.LineVector(diagram.XY(radius_SUN - (self.W_SUN // 2), center_SUN.y), diagram.VectorPolar(delta_SUNSHADE, -180),
+                                 color=diagram.C(outline=self.C_SUN.outline),
+                                 arrow=diagram.A(),
+                                 )
+        yield diagram.ArcCircleCenter(center_SUN, radius_SUN - delta_SUNSHADE,
+                                      rng=self.SUNSHADE_RANGE,
+                                      color=diagram.C(outline=self.C_SUN.outline),
+                                      width=self.W_SUN, dash=diagram.D(6, 4),
+                                      )
+
         # Rest
         rest_XY = diagram.XY((radius_SUN + self.W_SUN), cheight // 2)
         rest_width = cwidth - rest_XY.x
@@ -163,7 +177,7 @@ class Diagram_SolarSystem(diagram.Diagram):
         yield diagram.CircleCenter(center_MOON, radius_MOON,
                                    color=self.C_MOON, colorActive=self.C_MOON.reverse,
                                    width=self.W_MOON,
-                                   tags=['planet', 'planet:secondary'])
+                                   tags=['planet', 'planet:secondary']), 'moon'
         yield diagram.Text(center_MOON - diagram.VectorH(int(1.5 * radius_MOON)), 'Luna',
                            color=diagram.C(fill='white'),
                            anchor=CP.E, font=('TkHeadingFont', 12),
@@ -303,7 +317,7 @@ class ListFrame_Inner(FrameStateful):
 
 
 class ListFrame_Outer_Label(FrameUnlabelled):
-    layout = tk.HORIZONTAL
+    layout = HORIZONTAL
 
     def setup_widgets(self):
         self.cbL = Checkbox(self, label='').putIgnoreState()
@@ -330,7 +344,7 @@ class ListFrame_Outer(FrameStateful):
 
 
 class ListFrame_Lists(FramePaned):
-    layout = tk.HORIZONTAL
+    layout = HORIZONTAL
     # Do not grow RO widget
     pweigths = {
         'lstRO': 0,
@@ -351,7 +365,7 @@ class ListFrame_Lists(FramePaned):
 
 
 class ListFrame__Actions(FrameUnlabelled):
-    layout = tk.HORIZONTAL
+    layout = HORIZONTAL
 
     def setup_widgets(self):
         self.op1 = Button(self, label='Op1',
@@ -512,11 +526,45 @@ class NB_Child_Simple(FrameUnlabelled):
         w.change(image=self.wimage(imgname))
 
 
-class NB_Child_Complex(NotebookUniform):
-    tabids = {f'TC{d}': f'Tab Complex {d}' for d in range(5)}
+class NB_Child_Complex_NB(NotebookUniform):
+    tabids = {f'TC{d}': f'Tab Complex {d}' for d in range(7)}
 
-    def setup_tab(self, tid: str, tname: str):
-        return NB_Child_Simple(self, label=tid)
+    def setup_tab(self, tid: str, tname: str, *, labelPrefix):  # type: ignore[override]
+        return Scrolled(self, NB_Child_Simple, label=f'{tid} @ {labelPrefix}')
+
+
+class NB_Child_Complex(FramePaned):
+    layout = HORIZONTAL
+    pweigth = 0  # Keep defaults sizes uniform
+
+    def setup_widgets(self):
+        self.sidel = NB_Child_Complex_NB(self, traversalWraparound=False,
+                                         tabArgs={'image': random.choice(self.wroot.images_builtin)},
+                                         labelPrefix='Top',
+                                         styleID='TabCenterTop|TabLarge')
+        self.sider = NB_Child_Complex_NB(self, traversalWraparound=True,
+                                         tabArgs={'image': random.choice(self.wroot.images_builtin)},
+                                         labelPrefix='Bottom',
+                                         styleID='TabCenterBottom|TabLarge')
+
+    def setup_adefaults(self):
+        nb = self.sidel
+        # logger.debug('Change top widget:')
+        # logger.debug('   Default: %s', nb.torder_get())
+        # nb.torder_change(disable=['TC2'])
+        # logger.debug('  Disabled: %s', nb.torder_get())
+        # nb.torder_change(enable=['TC2'])
+        # logger.debug('Re-Enabled: %s', nb.torder_get())
+        # nb.torder_change(disable=['TC1', 'TC3'],
+        #                  hide=['TC2'])
+        # nb.insert(tk.END, nb.wtabs['TC0'].widget)
+        # nb.insert(0, nb.wtabs['TC4'].widget)
+        # logger.debug('     Final: %s', nb.torder_get())
+        nb.torder = NotebookTabOrder(
+            shown=('TC1', 'TC4', 'TC0', 'TC3'),
+            disabled={'TC1', 'TC3'},
+        )
+        # logger.debug('       Set: %s', nb.torder_get())
 
 
 class NB_Child_Timeout(FrameLabelled):
@@ -668,7 +716,7 @@ class NB_Child_TimeoutIdle_Chain(FrameLabelled):
 
 
 class NB_Child_Timeouts(FrameUnlabelled):
-    # layout = tk.HORIZONTAL
+    # layout = HORIZONTAL
 
     def setup_widgets(self):
         self.timeout = NB_Child_Timeout(self)
@@ -686,7 +734,7 @@ class NB_Child_Interval(FrameUnlabelled):
 
     def setup_widgets(self):
         self.txt_lbl = Label(self, label='Count (1s)')
-        self.txt = EntryRaw(self, justify=CP.center, readonly=True)
+        self.txt = EntryRaw(self, justify=Justification.Center, readonly=True)
         self.stateScheduled = Checkbox(self, label='Scheduled?', readonly=True)
         self.state_on = Button(self, label='ON')
         self.state_off = Button(self, label='OFF')
@@ -740,7 +788,7 @@ class NB_Child_RateLimiter(FrameUnlabelled):
     def setup_widgets(self):
         self.txt_lbl = Label(self, label='Now')
         self.txt = EntryRaw(self, readonly=True,
-                            justify=CP.center, width=30)
+                            justify=Justification.Center, width=30)
         self.hit_lbl = Label(self, label='Hit me Hard!\nCount will only change once per second.\nThe timings are not perfect.')
         self.buttonHit = Button(self, label='HIT')
         self.stateRL = Checkbox(self, label='Rate Limited?', readonly=True)
@@ -931,7 +979,7 @@ class NB_Complex_CheckboxNested(CheckboxFrame):
                 },
             },
             'stateDefault': True,
-            'layoutCheckboxes': [tk.HORIZONTAL, tk.VERTICAL, 'R1,x'],
+            'layoutCheckboxes': [HORIZONTAL, VERTICAL, 'R1,x'],
             'labelsCheckboxes': {
                 ('NN1(0)',): 'NN1 [[0]]',
                 **{
@@ -964,21 +1012,61 @@ class NB_Complex_CheckboxNested(CheckboxFrame):
         self._bState = new_state
 
 
-class NB_Complex_RadioList(RadioFrame):
-    def __init__(self, *args, **kwargs):
-        kwargs.update({
-            'stateRadio': StaticMap(
-                {
-                    'S#%d' % n: str(n)
-                    for n in range(random.randint(5, 11))
-                }, defaultValue='3',
-            ),
-        })
-        super().__init__(*args, **kwargs)
+class NB_Complex_RadioFrame(FrameUnlabelled):
+    wstate_single = False
+    layout = 'xE'
+
+    def setup_widgets(self):
+        sr_l = StaticMap(
+            {
+                'S#%d' % n: str(n)
+                for n in range(random.randint(5, 11))
+            }, defaultValue=str(random.choice(range(5))),
+        )
+        sr_r = StaticMap(
+            {
+                'S#%d' % n: str(n)
+                for n in range(random.randint(5, 11))
+            }, defaultValue=str(random.choice(range(5))),
+        )
+        self.b_l = Button(self, label='Random Left')
+        self.b_r = Button(self, label='Random Right')
+        self.rf_l = RadioFrameLabelled(self, stateRadio=sr_l, label='Left')
+        self.rf_r = RadioFrameStateful(self, stateRadio=sr_r, label='Right')
+
+    def setup_layout(self, layout):
+        self.pgrid_r(*self.widgets_class(Button),  # Buttons
+                     weight=0)
+
+    def setup_adefaults(self):
+        self.b_l.onClick = self.onRandom_RadioFrameLabelled
+        self.b_r.onClick = self.onRandom_RadioFrameStateful
+
+    def onRandom_RadioFrameLabelled(self):
+        twidget = self.rf_l
+        choice = random.choice(twidget.variable_raw.lall())
+        logger.debug('Random @ RFL(%s): %s', twidget, choice)
+        twidget.wstate = choice
+
+    def onRandom_RadioFrameStateful(self):
+        twidget = self.rf_r
+        choice = random.choice(twidget.variable_raw.lall())
+        logger.debug('Random @ RFS(%s): %s', twidget, choice)
+        twidget.wstate = WState(
+            random.choice([True, False]),
+            choice,
+        )
+
+    def onRandomW(self, twidget):
+        def onRandomW():
+            choice = random.choice(twidget.variable_raw.lall())
+            logger.debug('Random @ %s: %s', twidget, choice)
+            twidget.wstate = choice
+        return onRandomW
 
 
 class NB_Child_Checkboxen(FrameStateful):
-    layout = tk.VERTICAL
+    layout = VERTICAL
     wstate_single = 'cb'
 
     def setup_widgets(self, *, cbReadonly: bool):
@@ -986,12 +1074,27 @@ class NB_Child_Checkboxen(FrameStateful):
         self.cb = Checkbox(self, label=f'Checkbox {cb_str}', readonly=cbReadonly)
 
 
+class NB_Complex_ChechboxRORW_Reversed(FrameStateful):
+    label = 'Reversed'
+    layout = HORIZONTAL
+
+    def setup_widgets(self, *, rw, ro):
+        self.rw = NB_Child_Checkboxen(self, label='ReadWrite (Frame=ReadWrite)',
+                                      cvariable=rw, cstateReadonly=False,
+                                      cbReadonly=False, cstateReversed=True)
+        self.ro = NB_Child_Checkboxen(self, label='ReadOnly (Frame=ReadOnly)',
+                                      cvariable=ro, cstateReadonly=True,
+                                      cbReadonly=True, cstateReversed=True)
+
+
 class NB_Complex_ChechboxRORW(FrameUnlabelled):
-    layout = 'R2,x'
+    layout = 'R2,1,x'
 
     def setup_widgets(self):
         self.rw = NB_Child_Checkboxen(self, label='ReadWrite', cbReadonly=False)
         self.ro = NB_Child_Checkboxen(self, label='ReadOnly', cbReadonly=True)
+        self.rev = NB_Complex_ChechboxRORW_Reversed(self, labelAnchor=CP.S,
+                                                    rw=self.rw.cstate, ro=self.ro.cstate)
         self.read = Button(self, label='Check')
         self.toggle_rw = Button(self, label='Toggle RW "alternate"')
 
@@ -1010,7 +1113,8 @@ class NB_Complex_ChechboxRORW(FrameUnlabelled):
         logger.debug('   RO: %s', self.ro.cb.gstate)
 
     def onToggleRW(self):
-        self.rw.cb.gstate = GuiState(alternate=not self.rw.cb.gstate.alternate)
+        for w in (self.rw, self.rev.rw):
+            w.cb.gstate = GuiState(alternate=not w.cb.gstate.alternate)
 
 
 class NB_Complex_Validation_Spinbox(FrameLabelled):
@@ -1137,7 +1241,7 @@ class NB_Complex_Validation_Combobox(FrameLabelled):
 
 
 class NB_Complex_Validation(FrameUnlabelled):
-    layout = tk.HORIZONTAL
+    layout = HORIZONTAL
 
     def setup_widgets(self):
         lim = LimitBounded(5, 7, fn=fn.valNumber)
@@ -1170,12 +1274,12 @@ class NB_Complex_ValidationNew(FrameLabelled):
                          styleID='Valid')
         self.lComboboxSingle = Label(self, label='Combobox [Single]:', styleID='Valid')
         self.cs1 = ComboboxN(self, vsingle,
-                             styleID='Big|Valid')
+                             styleID='Big|Valid').putTooltip()
         self.cs2 = ComboboxN(self, vsingle, readonly=False,
                              styleID='Big|Valid')
         self.lComboboxMap = Label(self, label='Combobox [Map]:', styleID='Valid')
         self.cm1 = ComboboxN(self, vsmap,
-                             styleID='Big|Valid')
+                             styleID='Big|Valid').putTooltip()
         self.cm2 = ComboboxN(self, vsmap, readonly=False,
                              styleID='Big|Valid')
         self.lComboboxLimit = Label(self, label='Combobox [Limit]:', styleID='Valid')
@@ -1206,7 +1310,7 @@ class NB_Complex_ValidationNew(FrameLabelled):
 
 
 class NB_Complex_FramePaned(FramePaned):
-    layout = tk.HORIZONTAL
+    layout = HORIZONTAL
 
     def setup_widgets(self):
         self.left = ListFrame_Inner(self, label='Left Side',
@@ -1218,7 +1322,8 @@ class NB_Complex_Canvas(FrameUnlabelled):
     layout = 'R1,x'
 
     def setup_widgets(self):
-        self.cv = Canvas(self, Diagram_SolarSystem())
+        self.cv = Canvas(self, Diagram_SolarSystem(),
+                         saveElements=True)
         self.redraw = Button(self, label='Redraw')
         self.redraw_force = Button(self, label='Redraw (Force)')
         self.toggle_state = Button(self, label='Toggle')
@@ -1237,8 +1342,13 @@ class NB_Complex_Canvas(FrameUnlabelled):
 
     def onClickCanvasElement(self, event):
         widget = event.widget
-        clocation = diagram.XY(widget.canvasx(event.x), widget.canvasy(event.y))
-        logger.debug('Clicked %s @ Element %d', clocation, widget.eselection())
+        eselection = widget.eselection()
+        logger.debug('Clicked %s @ #%d', widget.exy(event), eselection)
+        if widget.hasElements:
+            if emarker := widget.itemMarker(eselection):
+                logger.debug('| Marker: `%s`', emarker)
+            if eelement := widget.item(eselection):
+                logger.debug('| Element: %s', eelement)
 
     def onToggleCanvasState(self, event=None):
         self.cv.gstate = GuiState(enabled=not self.cv.gstate.enabled)
@@ -1261,6 +1371,155 @@ class NB_Complex_Canvas(FrameUnlabelled):
                     logger.debug('  >>> %r: %s', evalue, ' '.join(ekeys))
 
 
+class NB_Complex_Scrolled_InnerLabels(FrameUnlabelled):
+    def setup_widgets(self, *, count: int):
+        widgets = {}
+        for n in range(1, count + 1):
+            widgets[f'l:{n}'] = Label(self, label=f'Label @ {n}')
+        return widgets
+
+
+class NB_Complex_Scrolled_Inner(FrameUnlabelled):
+    layout = 'R1,x,1'
+
+    def setup_widgets(self, *, count: int, ilayout=AUTO):
+        self.top = EntryRaw(self, justify=Justification.Center)
+        self.lbls = NB_Complex_Scrolled_InnerLabels(self, layout=ilayout,
+                                                    count=count)
+        self.bot = EntryRaw(self, justify=Justification.Center)
+
+    def setup_layout(self, layout):
+        self.pgrid_r(*self.widgets_class(EntryRaw),
+                     weight=0)
+        for w in self.widgets_class(EntryRaw):
+            w.grid(sticky=tk.EW)
+
+    def setup_defaults(self):
+        self.wstate = {
+            'top': 'Top Entry',
+            'bot': 'Bottom Entry',
+        }
+
+
+class NB_Complex_Scrolled(FrameStateful):
+    layout = 'R1,2,1,1,2'
+    label = 'Scrolled'
+    layout_autoadjust = True
+
+    def setup_widgets(self):
+        count = random.choice([15 * (2 * n + 4) for n in range(2)])
+        count_massive = 5 * count
+        assert count_massive % 30 == 0
+
+        self.fh_lbl = Label(self, label='Scrolled: Horizontal (row weight=0)',
+                            styleID='Title')
+        self.fv_lbl = Label(self, label='Scrolled: Vertical (row weight=1)',
+                            styleID='Title')
+        self.fb_lbl = Label(self, label='Scrolled: Both (row weight=1)',
+                            styleID='Title')
+        self._s_h = SeparatorH(self)
+        self.fh = Scrolled(self, NB_Complex_Scrolled_Inner, count=count,
+                           scrollHorizontal=True,
+                           ilayout=HORIZONTAL)
+        self.fv = Scrolled(self, NB_Complex_Scrolled_Inner, count=count,
+                           scrollVertical=True,
+                           ilayout=VERTICAL)
+        self.fb = Scrolled(self, NB_Complex_Scrolled_Inner, count=count_massive,
+                           scrollHorizontal=True, scrollVertical=True,
+                           ilayout='30xE')
+
+    def setup_layout(self, layout):
+        self.pgrid_r(*self.widgets_class(Label),
+                     weight=0)
+        for w in self.widgets_class(Label):
+            w.grid(sticky=tk.EW)
+
+    def setup_adefaults(self):
+        # Make sure `widgets_except` works properly with proxy widgets
+        assert self.fh.wproxy in self.widgets_except(self.fv, self.fb)
+        assert self.fh not in self.widgets_except(self.fh)
+
+
+class NB_Complex_ValidationAdvanced(FrameUnlabelled):
+    layout = 'R4,2,2,4'
+
+    def setup_widgets(self):
+        vtt_a = self.varSpec(var.Limit, LimitUnbounded(Fraction(1, 2), Fraction(5), default='0.75',
+                                                       imin=False, fn=fn.valFraction),
+                             name='vtt_a')
+        vtt_oi = self.varSpec(var.Limit, LimitUnbounded('-10/3', '10/3', default='0',
+                                                        fn=fn.valFraction),
+                              name='vtt_oi')
+        vstep = self.varSpec(var.Limit, LimitBounded(10, 60, step=5, imax=False, fn=fn.valNumber),
+                             name='vstep')
+        vdynamic = self.varSpec(var.Limit, vstep.container,
+                                name='vdynamic')
+
+        self.lDynamic = Label(self, label='Dynamic:\nValid=Labels Valid=True',
+                              styleID='Valid')
+        self.bDynamic = Button(self, label='Shuffle')
+        self.cdynamic_vl = ComboboxN(self, vdynamic, validation=ComboboxN.ValidationLabels,
+                                     styleID='Big|Valid')
+        self.cdynamic_v = ComboboxN(self, vdynamic, validation=True,
+                                    styleID='Big|Valid')
+        self.lSimple = Label(self, label=f'Tooltip: always\n{vtt_a.container}',
+                             styleID='Valid')
+        self.esimple = EntryN(self, vtt_a,
+                              validation=VSettings(fn=self.onValid_Simple, fnSimple=False),
+                              styleID='Valid')
+        self.tsimple = TooltipSimple(self.esimple, tt_position=CP.N)
+        self.lTinvalid = Label(self, label=f'Tooltip: var.ValidationLimit\n{vtt_oi.container}',
+                               styleID='Valid')
+        self.etinvalid = EntryN(self, vtt_oi,
+                                validation=var.ValidationLimit,
+                                styleID='Valid').putTooltip(tt_position=CP.N)
+        self.lStepS = Label(self, label=f'Spinbox\n{vstep.container}',
+                            styleID='Valid')
+        self.lStepC = Label(self, label=f'ComboboxN\n{vstep.container}',
+                            styleID='Valid')
+        self.sstep = SpinboxN(self, vstep, readonly=False,
+                              styleID='Big|Valid').putTooltip()
+        self.cstep = ComboboxN(self, vstep, readonly=True,
+                               styleID='Big|Valid').putTooltip()
+
+    def setup_layout(self, layout):
+        self.pgrid_c(*self.widgets_class(Label), *self.widgets_class(Button),
+                     weight=0)
+
+    def setup_adefaults(self):
+        self.bDynamic.onClick = self.onShuffleDynamic
+        self.setup_gstate_valid(nowarn=True,
+                                childMatch=self.widgets_class(Label))
+
+    def onShuffleDynamic(self):
+        all_values = self.gvar('vstep').lall()
+        all_count = len(all_values)
+        new = random.sample(all_values, k=random.randrange(all_count // 3, 2 * all_count // 3))
+        logger.debug('All=%d Choices=%d', all_count, len(new))
+        for w in (self.cdynamic_vl, self.cdynamic_v):
+            w.change(values=sorted(new), valuesState=w == self.cdynamic_vl)
+
+    def onValid_Simple(self, vstate, why):
+        assert why is not None  # Needs fnSimple=False
+        wvalidation = self.esimple
+        assert wvalidation == why.widget
+        wtooltip = self.tsimple
+        vspec = wvalidation.variable.container
+        vfn = vspec.fn
+        vstate_raw = vfn(vstate.label)
+        wtooltip.wstate = {
+            'mtitle': f'Validation For {wvalidation}',
+            'message': dedent(f'''
+                VState: {vstate!r} | raw={vstate_raw!r}
+                Why: {why!r}
+                Variable: {vspec}
+                ---
+                Trigger TT: {wtooltip}
+            '''),
+        }
+        return vstate.valid
+
+
 class NB_Complex(Notebook):
     def setup_tabs(self):
         return {
@@ -1269,11 +1528,15 @@ class NB_Complex(Notebook):
             'lc': Notebook.Tab('ListboxControl', NB_Complex_ListboxSet(self)),
             'cl': Notebook.Tab('CheckboxFrame', NB_Complex_CheckboxList(self)),
             'cn': Notebook.Tab('CheckboxFrame\nNested', NB_Complex_CheckboxNested(self)),
-            'rl': Notebook.Tab('RadioFrame', NB_Complex_RadioList(self)),
+            'rl': Notebook.Tab('RadioFrame', NB_Complex_RadioFrame(self)),
+            'scrolled': Notebook.Tab('Scrolled', NB_Complex_Scrolled(self)),
             'cb_rorw': Notebook.Tab('CB RO/RW', NB_Complex_ChechboxRORW(self)),
             'fpaned': Notebook.Tab('FramePaned', NB_Complex_FramePaned(self)),
             'validation': Notebook.Tab('Validation Old', NB_Complex_Validation(self)),
-            'vnew': Notebook.Tab('Validation New', NB_Complex_ValidationNew(self, labelAnchor=CP.N, styleID='Valid')),
+            'vnew': Notebook.Tab('Validation New', NB_Complex_ValidationNew(self, labelAnchor=CP.N,
+                                                                            styleID='Valid')),
+            'vadv': Notebook.Tab('Validation\nAdvanced', NB_Complex_ValidationAdvanced(self,
+                                                                                       styleID='Valid')),
             'canvas': Notebook.Tab('Canvas', NB_Complex_Canvas(self)),
         }
 
@@ -1294,7 +1557,7 @@ class NB(Notebook):
             'trl': Notebook.Tab('RateLimiter', NB_Child_RateLimiter(self)),
             'td': Notebook.Tab('Dialogues', NB_Child_Dialog(self),
                                image=self.wimage('info-msgbox-s16'), labelPosition=True),  # Only image
-            'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self, styleID='TabCenter|TabLarge'),
+            'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self),
                                image=self.wimage('info-s16'), labelPosition=CP.W),  # "Reverse" labelPosition
             't1': Notebook.Tab('Tab 1', NB_Child_Simple(self, label='Tab 1')),
             't2': Notebook.Tab('Tab 2', NB_Child_Simple(self, label='Tab 2')),
@@ -1494,19 +1757,54 @@ class NB_Center_Radio(FrameUnlabelled):
 class NB_Center_RadioFrames_Inner(FrameRadio):
     wstate_single = 'inner'
 
-    def setup_widgets(self, n):
+    def setup_widgets(self, n, ro=False):
         self.lbl = Label(self, label=f'Inner\nRadio Frame\n{n}')
-        self.inner = ListFrame_Inner(self, label=n)
+        self.inner = ListFrame_Inner(self, label=n, cstateReadonly=ro)
 
 
-class NB_Center_RadioFrames_FInner(FrameStateful):
-    layout = tk.HORIZONTAL
+class NB_Center_RadioFrames_FL(FrameLabelled):
+    layout = 'R1,x,x'
+
+    def setup_widgets(self, rvar):
+        lall = rvar.lall()
+        widgets = {}
+        widgets['toggle'] = Button(self, label='Toggle Inner FrameStateful')
+        for lbl in lall:
+            widgets[f'r:{lbl}'] = Radio(self, label=f'Frame {lbl}',
+                                        variable=rvar, value=lbl)
+        for lbl in lall:
+            widgets[f'f:{lbl}'] = NB_Center_RadioFrames_Inner(self, label=f'Frame {lbl}',
+                                                              rvariable=rvar, rvalue=lbl,
+                                                              n=lbl, ro=True)
+        return widgets
+
+    def setup_layout(self, layout):
+        toggle = self.widgets['toggle']
+        self.pgrid_r(toggle, *self.widgets_class(Radio),
+                     weight=0)
+        toggle.grid(sticky=tk.EW)
+        for w in self.widgets_class(Radio):
+            w.layout_padable = False
+            w.grid(sticky=tk.W,
+                   padx=(SStyle.Size_PadWidget_FrameSeparator, 0))
+
+    def setup_adefaults(self):
+        self.widgets['toggle'].onClick = self.onToggle
+
+    def onToggle(self):
+        for w in self.widgets_class(NB_Center_RadioFrames_Inner):
+            w.inner.cstate_widget.toggle()
+
+
+class NB_Center_RadioFrames_FS(FrameStateful):
+    layout = HORIZONTAL
 
     def setup_widgets(self, rvar):
         widgets = {}
         for lbl in rvar.lall():
-            widgets[f'f:{lbl}'] = NB_Center_RadioFrames_Inner(self, label=f'Frame {lbl}',
+            widgets[f'f:{lbl}'] = NB_Center_RadioFrames_Inner(self, label=f'Frame {lbl} (Reversed)',
                                                               rvariable=rvar, rvalue=lbl,
+                                                              rstateReversed=True,
                                                               n=lbl)
         return widgets
 
@@ -1515,15 +1813,19 @@ class NB_Center_RadioFrames(FrameUnlabelled):
     layout = 'x2E'
 
     def setup_widgets(self):
-        rvar = self.varSpec(var.StaticList,
-                            StaticList(('LeftSide', 'RightSide'), defaultIndex=0),
-                            name='frame')
+        srvar = self.varSpec(var.StaticList,
+                             StaticList(('LeftSide', 'RightSide'), defaultIndex=0),
+                             name='frame:s')
+        lrvar = self.varSpec(var.StaticList,
+                             StaticList(('LeftSide', 'RightSide'), defaultIndex=0),
+                             name='frame:l')
 
         widgets = {}
-        for lbl in rvar.lall():
-            widgets[f'r:{lbl}'] = Radio(self, label=f'Radio {lbl}',
-                                        variable=rvar, value=lbl)
-        widgets['finner'] = NB_Center_RadioFrames_FInner(self, label='Stateful', rvar=rvar)
+        for lbl in srvar.lall():
+            widgets[f'r:{lbl}'] = Radio(self, label=f'Radio Stateful: {lbl}',
+                                        variable=srvar, value=lbl)
+        widgets['fs'] = NB_Center_RadioFrames_FS(self, label='Stateful', rvar=srvar)
+        widgets['fl'] = NB_Center_RadioFrames_FL(self, label='Labelled', rvar=lrvar)
         return widgets
 
     def setup_defaults(self):
@@ -1743,7 +2045,7 @@ class NB_Center_SWin_Popup(SecondaryWindow):
 
 
 class NB_Center_SWin_Selection__Buttons(FrameUnlabelled):
-    layout = tk.HORIZONTAL
+    layout = HORIZONTAL
 
     def setup_widgets(self, NAMES):
         widgets = {}
@@ -1841,7 +2143,7 @@ class NB_Center_SWin(FrameStateful):
 
 
 class NB_Center_Tooltip(FrameUnlabelled):
-    layout = tk.VERTICAL
+    layout = VERTICAL
 
     def setup_widgets(self):
         self.txt = Label(self, label='This label has a tooltip')
@@ -1861,7 +2163,7 @@ class NB_Center(Notebook):
             'label_align:sl': Notebook.Tab('Label Align:SL', NB_Center_LabelAlignSL(self)),
             'label_align:ml': Notebook.Tab('Label Align:ML', NB_Center_LabelAlignML(self)),
             'radio': Notebook.Tab('Radio', NB_Center_Radio(self)),
-            'frame:radio': Notebook.Tab('RadioFrames', NB_Center_RadioFrames(self)),
+            'frame:radio': Notebook.Tab('FrameRadio', NB_Center_RadioFrames(self)),
             'ltml': Notebook.Tab('LTML', LTMLEditor(self)),
             'el': Notebook.Tab('Event Loop', NB_Center_EventLoop(self)),
             'swin': Notebook.Tab('Secondary Windows', NB_Center_SWin(self)),
@@ -1869,7 +2171,7 @@ class NB_Center(Notebook):
         }
 
     def setup_adefaults(self):
-        self.wselect('tooltip')
+        self.wselect('frame:radio')
 
 
 class RW(RootWindow):
@@ -1877,7 +2179,8 @@ class RW(RootWindow):
     # "TNotebook.tabplacement" is undocumented and buggy, should start with the "opposite" of "tabposition"
     # - See https://stackoverflow.com/a/76007959/12287472
     styleIDs = {
-        'TabCenter.TNotebook': {'tabposition': tk.N},
+        'TabCenterTop.TNotebook': {'tabposition': tk.N},
+        'TabCenterBottom.TNotebook': {'tabposition': tk.S},
         'TabLarge.TNotebook': {}, 'TabLarge.TNotebook.Tab': {'padding': (20, 5)},
         'TabV_W.TNotebook': {'tabposition': tk.W + tk.N, 'tabplacement': tk.N + tk.EW},
         'ReadonlyEmphasis.TCheckbutton': {},
@@ -1901,11 +2204,12 @@ class RW(RootWindow):
             'arrowsize': '20',
         },
         'Out.TLabelframe': {'labeloutside': True},
-        'Valid.TSpinbox': {
-            'arrowcolor': 'blue',
-        },
-        'Valid.TCombobox': {
-            'arrowcolor': 'blue',
+        # Valid
+        **{
+            f'Valid.{sid}': {
+                'arrowcolor': 'blue',
+            }
+            for sid in ('TSpinbox', 'TCombobox', 'TSpinbox')
         },
         'Valid.TEntry': {
             'foreground': 'blue',
@@ -1971,13 +2275,14 @@ class RW(RootWindow):
         logger.debug('Setup bindings')
         BindingGlobal(self, '<F4>', self.debug,
                       immediate=True, description='Debug')
-        BindingGlobal(self, '<Shift-Button-2>', lambda e: self.debugInfo(e.widget),
-                      immediate=True, description='Debug GUI for Current Widget')
-        BindingGlobal(self, '<Control-Shift-Button-2>', lambda e: self.debugInfo(e.widget, parent=True),
-                      immediate=True, description='Debug GUI on Parent Widget')
-        # This is only for "debuggier" debug sessions.
-        # BindingGlobal(self, '<Button-3>', self.debugEvent,
-        #               immediate=True, description='Debug Events')
+        if __debug__:
+            BindingGlobal(self, '<Shift-Button-2>', fn.onDebugWidget,
+                          immediate=True, description='Debug GUI for Current Widget')
+            BindingGlobal(self, '<Control-Shift-Button-2>', fn.onDebugPWidget,
+                          immediate=True, description='Debug GUI on Parent Widget')
+            # This is only for "debuggier" debug sessions.
+            # BindingGlobal(self, '<Button-3>', self.debugEvent,
+            #               immediate=True, description='Debug Events')
         logger.debug('Setup traces')
         self.tracev('bool', self.onTraceBool, trace_initial=True)
         self.trace(self.onTraceRoot)
@@ -2056,31 +2361,6 @@ class RW(RootWindow):
         if event is not None:
             self.debugEvent(event)
 
-    def debugInfo(self, widget, *, parent: bool = False):
-        if parent:
-            if widget.wparent is None:
-                assert str(widget) == '.', f'Invalid widget parent: {widget!r} » {widget.wparent!r}'
-            widget = widget.wparent
-        else:
-            widget = widget
-        logger.debug('%s [%s]', widget, widget.__class__.__qualname__)
-        if widget is None:
-            return
-        logger.debug('| %s', widget.gstate)
-        if hasattr(widget, 'wlabel'):
-            logger.debug('| Label: %s', widget.wlabel)
-        if isinstance(widget, mixin.ContainerWidget):
-            logger.debug('| Widgets:')
-            for wname, w in widget.widgets.items():
-                logger.debug('| | %s: %s', wname, w.gstate)
-            if widget != widget.wroot:
-                wstate_lines = pformat(widget.wstate).splitlines()
-                logger.debug('State: %s', wstate_lines[0])
-                for ln in wstate_lines[1:]:
-                    logger.debug('     : %s', ln)
-        else:
-            logger.debug('| Widget: %r', widget.wstate)
-
     def debugEvent(self, event=None):
         logger.debug('E: %s', event)
         if __debug__:
