@@ -56,21 +56,22 @@ def do(layout_input: typing.Optional[str], amount: int) -> typing.Tuple[typing.O
         layout = LAYOUT_SYNONYMS.get(layout_input, layout_input)
         assert layout is not None, f'Invalid Layout: {layout_input}'
     if __debug__:
-        logger.debug('Final Layout: %s', layout)
+        logger.debug('Initial Layout: %s', layout)
     # Do it!
     if layout:
         direction_names = tuple(d.name for d in model.Direction)
         if layout.startswith(('x', 'X')):
             auto_separator = layout[0]
-            if layout[1:] in ('', *direction_names):
+            if layout[1:] in ('', *direction_names):  # TODO: assert
                 square = math.ceil(math.sqrt(amount))
-                logger.debug('Layout: Square (%d)', square)
+                logger.debug('  Layout: Square (%d)', square)
                 layout = layout.replace(auto_separator, '%d%s%d' % (square, auto_separator, square), 1)
-                logger.debug('      : %s', layout)
+                logger.debug('        : %s', layout)
+                # TODO: layout='%dx%d' (square, square)
         if layout.startswith(tuple(LAYOUT_MULTIPLE)):
             _type = layout[0]
             multiple_direction = LAYOUT_MULTIPLE[_type]
-            logger.debug('Layout: Multiples %s: %s', _type, multiple_direction)
+            logger.debug('  Layout: Multiples %s: %s', _type, multiple_direction)
             amounts = [parse_amount(v) for v in layout[1:].split(',')]
             args = multiple_direction.multiples(*amounts, amount=amount)
         elif 'x' in layout or 'X' in layout:
@@ -99,7 +100,7 @@ def do(layout_input: typing.Optional[str], amount: int) -> typing.Tuple[typing.O
             if grid_missing > 0 and not auto_force:
                 if grid_missing >= cols:
                     dcols = grid_missing // cols
-                    logger.debug('      : -%d Columns', dcols)
+                    logger.debug('        : -%d Columns', dcols)
                     cols -= dcols
                     grid_missing = rows * cols - amount
                     if __debug__:
@@ -108,16 +109,17 @@ def do(layout_input: typing.Optional[str], amount: int) -> typing.Tuple[typing.O
                             logger.warning('%s: Non-automatic layout being unsquared: %d cols', layout_input, dcols)
                 if grid_missing >= rows:
                     drows = grid_missing // rows
-                    logger.debug('      : -%d Rows', drows)
+                    logger.debug('        : -%d Rows', drows)
                     rows -= drows
                     grid_missing = rows * cols - amount
                     if __debug__:
                         if layout_input not in LAYOUT_SYNONYMS:
                             # This might be a spurious warning
                             logger.warning('%s: Non-automatic layout being unsquared: %d rows', layout_input, drows)
-            logger.debug('Layout: Automatic Grid (%d%s%d%s)[%+d]', rows, auto_separator, cols, auto_direction.name, grid_missing)
+            logger.debug('  Layout: Automatic Grid (%d%s%d%s)[%+d]', rows, auto_separator, cols, auto_direction.name, grid_missing)
             args = auto_direction.grid(rows, cols, amount=amount,
                                        auto_fill=not auto_force)
+            # TODO: layout='%dx%d' (rows, cols)
         container_matrix = None  # For debug
         if __debug__:
             try:
@@ -133,12 +135,14 @@ def do(layout_input: typing.Optional[str], amount: int) -> typing.Tuple[typing.O
                         for dcol in range(arg.columnspan):
                             container_matrix[arg.row + drow][arg.column + dcol] = idx
                 else:
-                    logger.debug('| %s' % arg)
+                    logger.debug('  | %s' % arg)
             if container_matrix is not None:
+                cnt = math.ceil(math.log10(amount))
                 for r in container_matrix:  # Print widget locations
                     assert isinstance(r, typing.Sequence)
-                    logger.debug('| %s', ' '.join(('x' * 2 if i is None else '%02d' % i for i in r)))
+                    logger.debug('  | %s', ' '.join(('x' * cnt if i is None else f'%0{cnt}d' % i for i in r)))
             args = iter(logged_args)  # ... and return a new iterator
+    logger.debug('Final Layout: %s', layout)
     return layout, args
 
 
