@@ -22,7 +22,7 @@ import logging
 from functools import singledispatchmethod
 import abc
 from enum import Enum
-from dataclasses import dataclass, field
+from dataclasses import dataclass, field, asdict as dc_asdict
 import math
 import numbers
 import tkinter as tk
@@ -515,6 +515,10 @@ class C:
         # Equivalent to using sentinel values as defaults, `None` is a valid value
         return C(fill=kwargs.get('fill', self.fill), outline=kwargs.get('outline', self.outline))
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}({", ".join("=".join((k, v)) for k, v in dc_asdict(self).items() if v is not None)})'
+
 
 @dataclass
 class A:
@@ -533,15 +537,50 @@ class A:
         d3: Arrow Head Shape: Distance normal from the line, from the outside
             edge to the trailing points.
         atStart: Include an arrow head on the start of the line.
-            Defaults to `False`.
+            Optional, defaults to `False` if no other ``at*`` settings are is
+            given.
         atEnd: Include an arrow head on the end of the line.
-            Defaults to `True`.
+            Optional, defaults to `True` if no other ``at*`` settings are is
+            given.
+        atBoth: Include an arrow head on start and end of the line.
+            Optional, to be used like this:
+
+            .. code:: python
+
+                A(True)
     '''
-    d1: int = 8
-    d2: int = 10
-    d3: int = 3
-    atStart: bool = False
-    atEnd: bool = True
+    d1: int
+    d2: int
+    d3: int
+    atStart: bool
+    atEnd: bool
+
+    def __init__(self, atBoth: typing.Optional[bool] = None, *,
+                 atStart: typing.Optional[bool] = None, atEnd: typing.Optional[bool] = None,
+                 d1: int = 8, d2: int = 10, d3: int = 3,
+                 ):
+        # Start/End
+        finalAtStart: bool
+        finalAtEnd: bool
+        if atBoth is None:
+            if atStart is None and atEnd is None:
+                # Default Values: `(atEnd=True)`
+                finalAtStart, finalAtEnd = False, True
+            else:
+                finalAtStart = atStart or False
+                finalAtEnd = atEnd or False
+        else:
+            finalAtStart, finalAtEnd = atBoth, atBoth
+        assert finalAtStart is not None
+        assert finalAtEnd is not None
+        self.atStart = finalAtStart
+        self.atEnd = finalAtEnd
+        # Shapes
+        self.d1 = d1
+        self.d2 = d2
+        self.d3 = d3
+        # Validation
+        assert (self.atStart, self.atEnd) != (False, False), f'{self}: Include arrow at least in one end'
 
 
 @dataclass
@@ -587,6 +626,10 @@ class Cap(Enum):
     ROUND = tk.ROUND
     '''The end describes a semicircle centered on the endpoint.'''
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
 
 class Join(Enum):
     '''Join settings.
@@ -601,6 +644,10 @@ class Join(Enum):
     MITER = tk.MITER
     '''The edges of the adjacent line segments are continued to meet at a sharp point.'''
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
 
 class SmoothAlgorithm(Enum):
     '''Line smoothing algorithm selection.
@@ -626,6 +673,40 @@ class SmoothAlgorithm(Enum):
     information.
     '''
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
+
+class EllipseSection_Style(Enum):
+    '''Style selection for `EllipseSection`.
+
+    Check a visual representation of the style in ``Tk`` :tkinter_nmt:`NMT
+    create arc <create_arc.html>` or :tk:`canvas arc style <canvas.htm#M128>`
+    documentation.
+    '''
+    PIESLICE = tk.PIESLICE
+    '''Draw section as a pie slice.
+
+    This means the boundaries are defined by arc itself, plus two line
+    segments, between the oval center and each of the arc endpoints.
+    '''
+    CHORD = tk.CHORD
+    '''Draw section as a chord.
+
+    This means the boundaries are defined by the arc itself, plus a single line
+    segment between the arc endpoints.
+    '''
+    ARC = tk.ARC
+    '''Draw section as a simple arc.
+
+    This means drawing a line, just the arc itself. No area is defined.
+    '''
+
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
 
 @dataclass
 class Smooth:
@@ -664,6 +745,16 @@ class DiagramElement(abc.ABC):
         else:
             raise NotImplementedError(f'Unknown Type: {self!r}')
 
+    # This is an hack for the `Renderer_TkCanvas`, to pretend it's a tuple as `GeneratorElementT`
+    def __getitem__(self, key):
+        ''''''  # Internal, do not document
+        if key == 0:    # DiagramElement
+            return self
+        elif key == 1:  # marker
+            return None
+        else:
+            raise ValueError(f'{self}: Invalid Key: {key}')
+
 
 @dataclass
 class DiagramElementSingle(DiagramElement):
@@ -686,6 +777,17 @@ class DiagramElementMultiple(DiagramElement):
         self.items = tuple(items)
 
 
+GeneratorElementT = typing.Union[
+    DiagramElement,
+    typing.Tuple[DiagramElement, str]
+]
+'''Type-Checking variable type for `Diagram` drawing functions.
+
+Can be a `DiagramElement`, or a tuple of `DiagramElement` and `str`, used as a
+marker for the element in the renderer.
+'''
+
+
 # Support arbitrary layers?
 class Diagram(abc.ABC):
     '''Diagram generator class.
@@ -727,29 +829,44 @@ class Diagram(abc.ABC):
     Optional.
     '''
 
-    def setup_bg_b(self, *, cwidth: int, cheight: int) -> typing.Iterator[DiagramElement]:
+    def setup_bg_b(self, *, cwidth: int, cheight: int) -> typing.Generator[GeneratorElementT, typing.Optional[int], None]:
         '''Draw the Background Back layer.
 
-        Should be a generator creating a finite amount of `DiagramElement`.
+        Should be a generator creating a finite amount of `GeneratorElementT`.
+        Sends the element identifier (an integer).
+
+        .. note::
+            For debug binaries, sends `None` if the render had a problem. On
+            production binaries, this is an error.
         '''
         # Drawn only when size changes
-        return iter([])
+        yield from []
 
-    def setup_fg(self, *, cwidth: int, cheight: int) -> typing.Iterator[DiagramElement]:
+    def setup_fg(self, *, cwidth: int, cheight: int) -> typing.Generator[GeneratorElementT, typing.Optional[int], None]:
         '''Draw the Foreground layer.
 
-        Should be a generator creating a finite amount of `DiagramElement`.
+        Should be a generator creating a finite amount of `GeneratorElementT`.
+        Sends the element identifier (an integer).
+
+        .. note::
+            For debug binaries, sends `None` if the render had a problem. On
+            production binaries, this is an error.
         '''
         # Drawn always
-        return iter([])
+        yield from []
 
-    def setup_bg_f(self, *, cwidth: int, cheight: int) -> typing.Iterator[DiagramElement]:
+    def setup_bg_f(self, *, cwidth: int, cheight: int) -> typing.Generator[GeneratorElementT, typing.Optional[int], None]:
         '''Draw the Background Front layer.
 
-        Should be a generator creating a finite amount of `DiagramElement`.
+        Should be a generator creating a finite amount of `GeneratorElementT`.
+        Sends the element identifier (an integer).
+
+        .. note::
+            For debug binaries, sends `None` if the render had a problem. On
+            production binaries, this is an error.
         '''
         # Drawn only when size changes
-        return iter([])
+        yield from []
 
 
 # # External Diagram Elements
@@ -840,9 +957,6 @@ class MultiLine(DiagramElementSingle):
                         raise ValueError(f'{self}: "{tn}" distances have a range of ]0, 255]')
                     if tn != 'dash' and tv.offset != 0:
                         raise ValueError(f'{self}: "{tn}" does not support offsets, uses `dash` value')
-            if self.arrow:
-                if (self.arrow.atStart, self.arrow.atEnd) != (False, False):
-                    raise ValueError(f'{self}: When requesting arrows, include at least one')
 
 
 @dataclass
@@ -1169,8 +1283,96 @@ class Text(DiagramElementSingle):
             if self.angle < 0.0 or self.angle > 360.0:
                 raise ValueError(f'{self}: Invalid angle `{self.angle}`')
 
+
+@dataclass
+class EllipseSection(DiagramElementSingle):
+    '''Diagram Element: part of a larger `Ellipse`.
+
+    Represent a section of a simple `Ellipse`.
+
+    There is no Python documentation, see ``Tk`` :tk:`canvas common options
+    <canvas.htm#M99>` and :tk:`canvas arc options <canvas.htm#M125>`. See
+    also :tkinter_nmt:`NMT create arc <create_arc.html>` documentation.
+
+    This is a "low level" diagram element, you should use one of its higher
+    level elements: `ArcEllipse` or `PieEllipse`.
+
+    Args:
+        topleft: The top left vertex (lower X and Y coordinates).
+        botright: The bottom right vertex (higher X and Y coordinates).
+        style: Section style.
+        rng: Section range. The start and end angles for the section. The
+            angles are expressed in degrees, in a range between ``-360`` and
+            ``360``.
+        color: Colour for outline and fill area, see `C` for colour
+            specification.
+        width: Width for outlines, in pixels.
+            Optional, defaults to minimum line width.
+        dash: Dashed outline setting.
+            Optional, defaults to solid line.
+        colorActive: ``color`` for active elements, that is, on mouse hover.
+        widthActive: ``width`` for active elements, that is, on mouse hover.
+        dashActive: ``dash`` for disabled elements.
+            ``offset`` is not supported.
+        colorDisabled: ``color`` for disabled elements.
+        widthDisabled: ``width`` for disabled elements.
+        dashDisabled: ``dash`` for disabled elements.
+            ``offset`` is not supported.
+        tags: Sequence of tags to apply to the element.
+            Must not include any "internal tags", starting with ``:``.
+
+        .. note::
+
+            For `ARC <EllipseSection_Style.ARC>` style, there is no fill area.
+            This is checked in debug mode.
+    '''
+    topleft: XY
+    botright: XY
+    style: EllipseSection_Style
+    rng: typing.Tuple[numbers.Real, numbers.Real]  # Start, End
+    # Same as `Ellipse`
+    color: C = field(default_factory=C)
+    width: typing.Optional[int] = None
+    dash: typing.Optional[D] = None  # TODO: Accept (int, int), `D(*x)`
+    colorActive: C = field(default_factory=C)
+    widthActive: typing.Optional[int] = None
+    dashActive: typing.Optional[D] = None  # TODO: Accept (int, int), `D(*x)`
+    colorDisabled: C = field(default_factory=C)
+    widthDisabled: typing.Optional[int] = None
+    dashDisabled: typing.Optional[D] = None  # TODO: Accept (int, int), `D(*x)`
+    tags: typing.Sequence[str] = tuple()
+
+    if __debug__:
+        def __post_init__(self):
+            if self.style == EllipseSection_Style.ARC:
+                for c in ('color', 'colorActive', 'colorDisabled'):
+                    cv = getattr(self, c)
+                    if cv.fill is not None:
+                        logger.warning('%s: Arc does not support "fill" on `%s`', self, c)
+            astart, aend = self.rng
+            for aname, avalue in (('start', astart), ('end', aend)):
+                if avalue <= -360 or avalue >= 360:
+                    logger.warning('%s: range for rng:%s is `]-360, 360[`', self, aname)
+            assert 0 < self.extent < 360
+
+    @property
+    def extent(self) -> numbers.Real:
+        '''Calculate the extent of the section.
+
+        This is the delta between start and end.
+        Expressed in degrees, between ``0`` and ``360``.
+
+        .. note:
+
+            This is a bit more complex than a simple difference between start
+            and end angles, because both support negative values.
+        '''
+        astart, aend = self.rng
+        aextent = (360 + aend - astart) % 360
+        assert isinstance(aextent, numbers.Real), f'Invalid Number: {aextent!r}'
+        return aextent
+
 # TODO: Implement `Image`
-# TODO: Implement `EllipseArc` (enhance Ellipse instead?)
 
 
 # # # Complex Elements
@@ -1234,9 +1436,49 @@ def Line(p1: XY, p2: XY, **kwargs) -> MultiLine:
         p2: Second line segment vertex.
         kwargs: Passed to `MultiLine`.
     '''
+    assert 'points' not in kwargs, 'Use `MultiLine` directly'
     return MultiLine((p1, p2), **kwargs)
 
 
+def LineVector(p: XY, v: Vector, **kwargs) -> MultiLine:
+    '''Draw a single line segment, from a point following a vector.
+
+    Args:
+        p: First line segment vertex.
+        v: Vector to sum to ``p``, to obtain the second line segment vertex.
+        kwargs: Passed to `MultiLine`.
+    '''
+    return MultiLine((p, p + v), **kwargs)
+
+
+def ArcEllipse(**kwargs) -> EllipseSection:
+    '''Draw a simple ellipse arc, parallel to the axes.
+
+    The arguments are similar to `Ellipse`. Uses an `EllipseSection` with a
+    static style (`ARC <EllipseSection_Style.ARC>`).
+
+    Args:
+        kwargs: Passed to `EllipseSection`
+    '''
+    assert 'style' not in kwargs, 'Use `EllipseSection` directly'
+    kwargs['style'] = EllipseSection_Style.ARC
+    return EllipseSection(**kwargs)
+
+
+def PieEllipse(**kwargs) -> EllipseSection:
+    '''Draw a "pie" slice of a simple ellipse, parallel to the axes.
+
+    The arguments are similar to `Ellipse`. Uses an `EllipseSection` with a
+    static style (`PIESLICE <EllipseSection_Style.PIESLICE>`).
+
+    Args:
+        kwargs: Passed to `EllipseSection`
+    '''
+    assert 'style' not in kwargs, 'Use `EllipseSection` directly'
+    kwargs['style'] = EllipseSection_Style.PIESLICE
+    return EllipseSection(**kwargs)
+
+
 def RectangleCenter(c: XY, size_x: int, size_y: int, **kwargs) -> Rectangle:
     '''Draw a simple rectangle, from a center point and its size.
 
@@ -1249,6 +1491,7 @@ def RectangleCenter(c: XY, size_x: int, size_y: int, **kwargs) -> Rectangle:
         size_y: Size of rectangle, side parallel to Y axis.
         kwargs: Passed to `Rectangle`.
     '''
+    assert 'topleft' not in kwargs and 'botright' not in kwargs, 'Use `Rectangle` directly'
     delta = Vector(size_x // 2, size_y // 2)
     return Rectangle(c - delta, c + delta, **kwargs)
 
@@ -1276,11 +1519,27 @@ def EllipseCenter(c: XY, a: int, b: int, **kwargs) -> Ellipse:
         b: Semi-Axis parallel to Y.
         kwargs: Passed to `Ellipse`.
     '''
+    assert 'topleft' not in kwargs and 'botright' not in kwargs, 'Use `Ellipse` directly'
     axes_vector = Vector(a, b)
     return Ellipse(c - axes_vector, c + axes_vector, **kwargs)
 
 
-def CircleCenter(c: XY, radius: int, *args, **kwargs) -> Ellipse:
+def ArcEllipseCenter(c: XY, a: int, b: int, **kwargs) -> EllipseSection:
+    '''Draw a simple ellipse arc, from a center point to its axes.
+
+    Args:
+        c: Center point.
+        a: Semi-Axis parallel to X.
+        b: Semi-Axis parallel to Y.
+        kwargs: Passed to `ArcEllipse`.
+    '''
+    axes_vector = Vector(a, b)
+    assert 'topleft' not in kwargs and 'botright' not in kwargs, 'Use `ArcEllipse` directly'
+    return ArcEllipse(topleft=c - axes_vector, botright=c + axes_vector,
+                      **kwargs)
+
+
+def CircleCenter(c: XY, radius: int, **kwargs) -> Ellipse:
     '''Draw a circle, from a center point and its radius.
 
     Note that since integer coordinates are used, this might be off from
@@ -1293,3 +1552,18 @@ def CircleCenter(c: XY, radius: int, *args, **kwargs) -> Ellipse:
     '''
     assert radius == int(radius)
     return EllipseCenter(c, a=radius, b=radius, **kwargs)
+
+
+def ArcCircleCenter(c: XY, radius: int, **kwargs) -> EllipseSection:
+    '''Draw a circle arc, from a center point and its radius.
+
+    Note that since integer coordinates are used, this might be off from
+    the "real" radius by at most half pixel.
+
+    Args:
+        c: Center point
+        radius: Circle radius.
+        kwargs: Passed to `ArcEllipse`.
+    '''
+    assert radius == int(radius)
+    return ArcEllipseCenter(c, a=radius, b=radius, **kwargs)
