@@ -120,14 +120,42 @@ class Renderer_TkCanvas:
     happens when creating this object, all must go through the `redraw`
     function.
 
+    The possible values for the ``saveElements`` argument are:
+
+    - True: Save markers and element as a tuple
+    - False: Save only markers
+    - None: Disable saving elements or markers
+
     Args:
         canvas: The ``Tk`` canvas object to use.
             Should be an internal `Canvas`.
         diagram: The diagram to render.
+        saveElements: Save elements on the `eids` mapping, see above.
+            Defaults to `None`.
     '''
-    def __init__(self, canvas: 'Canvas', diagram: diagram.Diagram):
+    eids: typing.Optional[
+        typing.MutableMapping[int, typing.Union[
+            typing.Optional[str],
+            typing.Tuple[
+                typing.Optional[str],
+                diagram.DiagramElementSingle,
+            ],
+        ]]
+    ]
+    '''Map element ids to the corresponding `str` marker (if given), and
+    optionally including the `DiagramElement <diagram.DiagramElement>` as a
+    tuple.
+
+    Optional, this can be controlled with the ``saveElements`` parameter.
+    '''
+
+    def __init__(self, canvas: 'Canvas', diagram: diagram.Diagram, *,
+                 saveElements: typing.Optional[bool] = None,
+                 ):
         self._previous_size: typing.Tuple[int, int] = (-1, -1)
         self._e_clickelement: typing.Optional[str] = None
+        self.__saveElements: bool = saveElements is True
+        self.eids = None if saveElements is None else {}
         self.diagram = diagram
         self.cv = canvas
 
@@ -196,65 +224,137 @@ class Renderer_TkCanvas:
                 logger.debug('- Clean BG+FG (%d+%d)', len(cv.find_withtag(':bg')), len(cv.find_withtag(':fg')))
             assert set(cv.find_withtag(':bg || :fg')) == set(cv.find_all())
             cv.delete(tk.ALL)
+            if self.eids is not None:
+                self.eids.clear()
+                assert set(cv.find_all()) == set(self.eids.keys()), '{self.cv}: Bad `eids` manipulation: Clear all'
         elif redraw_fg:
             if __debug__:
                 logger.debug('- Clean FG (%d)', len(cv.find_withtag(':fg')))
             assert redraw_bg is False
             cv.delete(':fg')
+            if self.eids is not None:
+                for deid in cv.find(':fg'):
+                    if __debug__:
+                        if self.__saveElements is True:
+                            assert deid in self.eids, f'{self.cv}: Missing eid: {deid!r}'
+                    self.eids.pop(deid, None)
+                if __debug__:
+                    if self.__saveElements is True:
+                        assert set(cv.find_all()) == set(self.eids.keys()), '{self.cv}: Bad `eids` manipulation: Clear FG'
         else:
             # Skip redraw
             assert redraw_bg is False and redraw_fg is False
         drew_fg = False
         if __debug__:
             drew_bg_b, drew_bg_f = False, False
+            has_markers = False
+        eid: typing.Optional[int]
+        eid_marker: typing.Union[typing.Optional[str], typing.Tuple[typing.Optional[str], diagram.DiagramElementSingle]]
         # TODO: Add `:latest` tag everywhere
         if redraw_bg:
             if __debug__:
                 logger.debug('- Redraw BG:Back')
             tags_bg = [':bg', ':bg:b']
-            for eraw in self.diagram.setup_bg_b(cwidth=cwidth, cheight=cheight):
-                for element in eraw.iterate():
-                    eid = self.render(element, extratags=tags_bg)
-                    if eid is None:
-                        if __debug__:
-                            logger.warning('  - %s', element)
+            generator_bg_b = self.diagram.setup_bg_b(cwidth=cwidth, cheight=cheight)
+            try:
+                eid = None
+                while True:
+                    eraw = generator_bg_b.send(eid)
+                    assert isinstance(eraw, (tuple, diagram.DiagramElement))
+                    eraw_de, emarker = eraw[0], eraw[1]
+                    assert isinstance(eraw_de, diagram.DiagramElement)
+                    assert emarker is None or isinstance(emarker, str)
+                    if __debug__:
+                        has_markers = has_markers or emarker is not None
+                    for element in eraw_de.iterate():
+                        eid = self.render(element, extratags=tags_bg)
+                        if eid is None:
+                            if __debug__:
+                                logger.warning('  - %s', element)
+                            else:
+                                raise exception.InvalidRender(element)
                         else:
-                            raise exception.InvalidRender(element)
-                    else:
-                        logger.debug('  - %d: %s', eid, element)
-                        if __debug__:
-                            drew_bg_b = True
+                            logger.debug('  - %d: %s%s', eid, element, '' if emarker is None else f' [{emarker}]')
+                            if __debug__:
+                                drew_bg_b = True
+                            if self.eids is not None:
+                                if self.__saveElements:
+                                    eid_marker = (emarker, element)
+                                else:
+                                    eid_marker = emarker
+                                if eid_marker is not None:
+                                    self.eids[eid] = eid_marker
+            except StopIteration:
+                pass
         if redraw_fg:
             if __debug__:
                 logger.debug('- Redraw FG')
             tags_fg = [':fg']
-            for eraw in self.diagram.setup_fg(cwidth=cwidth, cheight=cheight):
-                for element in eraw.iterate():
-                    eid = self.render(element, extratags=tags_fg)
-                    if eid is None:
-                        if __debug__:
-                            logger.warning('  - %s', element)
+            generator_fg = self.diagram.setup_fg(cwidth=cwidth, cheight=cheight)
+            try:
+                eid = None
+                while True:
+                    eraw = generator_fg.send(eid)
+                    assert isinstance(eraw, (tuple, diagram.DiagramElement))
+                    eraw_de, emarker = eraw[0], eraw[1]
+                    assert isinstance(eraw_de, diagram.DiagramElement)
+                    assert emarker is None or isinstance(emarker, str)
+                    if __debug__:
+                        has_markers = has_markers or emarker is not None
+                    for element in eraw_de.iterate():
+                        eid = self.render(element, extratags=tags_fg)
+                        if eid is None:
+                            if __debug__:
+                                logger.warning('  - %s', element)
+                            else:
+                                raise exception.InvalidRender(element)
                         else:
-                            raise exception.InvalidRender(element)
-                    else:
-                        logger.debug('  - %d: %s', eid, element)
-                        drew_fg = True
+                            logger.debug('  - %d: %s%s', eid, element, '' if emarker is None else f' [{emarker}]')
+                            drew_fg = True
+                            if self.eids is not None:
+                                if self.__saveElements:
+                                    eid_marker = (emarker, element)
+                                else:
+                                    eid_marker = emarker
+                                if eid_marker is not None:
+                                    self.eids[eid] = eid_marker
+            except StopIteration:
+                pass
         if redraw_bg:
             if __debug__:
                 logger.debug('- Redraw BG:Front')
             tags_bg = [':bg', ':bg:f']
-            for eraw in self.diagram.setup_bg_f(cwidth=cwidth, cheight=cheight):
-                for element in eraw.iterate():
-                    eid = self.render(element, extratags=tags_bg)
-                    if eid is None:
-                        if __debug__:
-                            logger.warning('  - %s', element)
+            generator_bg_f = self.diagram.setup_bg_f(cwidth=cwidth, cheight=cheight)
+            try:
+                eid = None
+                while True:
+                    eraw = generator_bg_f.send(eid)
+                    assert isinstance(eraw, (tuple, diagram.DiagramElement))
+                    eraw_de, emarker = eraw[0], eraw[1]
+                    assert isinstance(eraw_de, diagram.DiagramElement)
+                    assert emarker is None or isinstance(emarker, str)
+                    if __debug__:
+                        has_markers = has_markers or emarker is not None
+                    for element in eraw_de.iterate():
+                        eid = self.render(element, extratags=tags_bg)
+                        if eid is None:
+                            if __debug__:
+                                logger.warning('  - %s', element)
+                            else:
+                                raise exception.InvalidRender(element)
                         else:
-                            raise exception.InvalidRender(element)
-                    else:
-                        logger.debug('  - %d: %s', eid, element)
-                        if __debug__:
-                            drew_bg_f = True
+                            logger.debug('  - %d: %s%s', eid, element, '' if emarker is None else f' [{emarker}]')
+                            if __debug__:
+                                drew_bg_f = True
+                            if self.eids is not None:
+                                if self.__saveElements:
+                                    eid_marker = (emarker, element)
+                                else:
+                                    eid_marker = emarker
+                                if eid_marker is not None:
+                                    self.eids[eid] = eid_marker
+            except StopIteration:
+                pass
         if drew_fg and len(cv.find_withtag(':bg:f')):
             if __debug__:
                 logger.debug('- Lower "FG" below "BG:Front"')
@@ -268,6 +368,8 @@ class Renderer_TkCanvas:
             drew_list = [lbl for lbl, b in drew_map.items() if b]
             logger.debug('Drew %d elements: %s', len(drew_list), ' '.join(drew_list))
             logger.debug('| Current Elements: %d', len(cv.find_all()))
+            if self.eids is None and has_markers:
+                logger.warning('%s: Markers without `saveElements`', self.cv)
         # TODO. Remove `:latest` tag, after logging the count
         # Setup Bindings
         e_clickelement = self.cv.tag_bind(tk.ALL, '<Button-1>', self.cv.onClickElement,
@@ -276,6 +378,55 @@ class Renderer_TkCanvas:
         self._previous_size = current_size
         self._e_clickelement = e_clickelement
 
+    def getMarker(self, eid: int) -> typing.Optional[str]:
+        '''Get a saved marker string, if exists.
+
+        Args:
+            eid: The element to consider.
+
+        Returns:
+            Return the marker `str`, if exists.
+        '''
+        if self.eids is None:
+            if __debug__:
+                logger.warning('%s: `getMarker` without `saveElements`', self.cv)
+            return None
+        else:
+            eid_marker = self.eids.get(eid)
+            if self.__saveElements:
+                assert isinstance(eid_marker, tuple) and len(eid_marker) == 2
+                emarker, _ = eid_marker
+                assert emarker is None or isinstance(emarker, str)
+                return emarker
+            else:
+                assert eid_marker is None or isinstance(eid_marker, str)
+                return eid_marker
+
+    def getElement(self, eid: int) -> typing.Optional[diagram.DiagramElement]:
+        '''Get a saved element, if exists.
+
+        Args:
+            eid: The element to consider.
+
+        Returns:
+            Return the `DiagramElement <diagram.DiagramElement>`, if saved.
+        '''
+        if self.eids is None:
+            if __debug__:
+                logger.warning('%s: `getElement` without `saveElements`', self.cv)
+            return None
+        else:
+            eid_marker = self.eids.get(eid)
+            if self.__saveElements:
+                assert isinstance(eid_marker, tuple) and len(eid_marker) == 2
+                _, element = eid_marker
+                assert isinstance(element, diagram.DiagramElement)
+                return element
+            else:
+                logger.warning('%s: `getElement` with `saveElements=False`', self.cv)
+                assert eid_marker is None or isinstance(eid_marker, str)
+                return None
+
     @singledispatchmethod
     def render(self, de: diagram.DiagramElement, *, extratags: typing.Sequence[str] = tuple()) -> int:
         '''Render a diagram element, with some internal ``extratags``.
@@ -293,7 +444,7 @@ class Renderer_TkCanvas:
         cls = de.__class__.__qualname__
         if __debug__:
             logger.critical('Missing render implementation for "%s"', cls)
-            return -1
+            return -1  # Not an error
         else:
             raise NotImplementedError(f'Canvas: Missing render implementation for {cls}')
 
@@ -399,3 +550,26 @@ class Renderer_TkCanvas:
                               active=de.colorActive,
                               disabled=de.colorDisabled)
         )
+
+    @render.register
+    def render_EllipseSection(self, de: diagram.EllipseSection, *, extratags: typing.Sequence[str] = tuple()) -> int:
+        assert all(t not in de.tags for t in extratags)  # Do not alias "real" tags
+        astart, aend = de.rng
+        return self.cv.create_arc(
+            de.topleft.tuple, de.botright.tuple,
+            style=de.style.value,
+            start=astart, extent=de.extent,
+            tags=(*de.tags, *extratags),
+            # Dash
+            **tkcanvas_dash(dash=de.dash,
+                            activedash=de.dashActive,
+                            disableddash=de.dashDisabled),
+            # Width
+            **tkcanvas_width(width=de.width,
+                             activewidth=de.widthActive,
+                             disabledwidth=de.widthDisabled),
+            # Colours
+            **tkcanvas_colors(de.color,
+                              active=de.colorActive,
+                              disabled=de.colorDisabled)
+        )
