@@ -9,6 +9,8 @@ import warnings
 import typing
 import sys
 import math
+from functools import wraps
+from fractions import Fraction
 from textwrap import dedent
 from pathlib import Path
 
@@ -16,6 +18,13 @@ import tkinter as tk
 import tkinter.filedialog
 import tkinter.messagebox
 
+try:
+    # Try to use nicer pretty-print, from Python 3.10
+    # | https://github.com/bitranox/pprint3x
+    from pprint3x import pformat
+except ImportError:
+    from pprint import pformat
+
 from . import model
 if typing.TYPE_CHECKING:
     from . import mixin
@@ -161,6 +170,37 @@ def bind_mousewheel(widget, up: typing.Callable, down: typing.Callable, **kwargs
         return NotImplementedError
 
 
+def scrollGenCommand(fn: typing.Callable[[float, float], None],
+                     why: typing.Optional[bool],
+                     what: str,
+                     fn_scrollstate: typing.Callable,
+                     ) -> typing.Callable[[float, float], None]:
+    ''''''  # Internal, do not document
+    # Wrap an existing scroll command function with automatic scroll state manipulation.
+
+    # This should take a ``set`` function defined in a scrollbar, and produces
+    # the function to attach to the ``xscrollcommand``/``yscrollcommand``
+    # parameter in corresponding scrolled widget.
+
+    # See `ScrolledWidget` and `Scrolled` for usage.
+    if why:
+        # Force Enable, don't change the state
+        return fn
+    else:
+        # Automatic, change the state as needed
+        @wraps(fn)
+        def scrollGenCommand(sposs: float, eposs: float) -> None:
+            # Defensive Programming, for older versions (make REALLY sure those are floats)
+            spos: float = float(sposs)
+            epos: float = float(eposs)
+            allvisible: bool = math.isclose(spos, 0.0) and math.isclose(epos, 1.0)
+            # if __debug__:
+            #     logger.debug('ScrollCommand[%s]: [%f , %f] = %s', what, spos, epos, allvisible)
+            fn_scrollstate(**{what: not allvisible})
+            return fn(spos, epos)
+        return scrollGenCommand
+
+
 def _filedialog_fts(filetypes, includeSupported, includeAll):
     ''''''  # Internal, do not document
     fts = [(t, ft.pattern) for t, ft in filetypes.items()]
@@ -489,6 +529,23 @@ def valNumber(string: str) -> typing.Optional[int]:
         return None
 
 
+def valFraction(string: str) -> typing.Optional[Fraction]:
+    '''Validate a number is a `Fraction <fractions.Fraction>` (ratio between
+    integers).
+
+    This ignores all possible whitespace, it is more lenient than the upstream
+    `Fraction <fractions.Fraction>` function.
+
+    Returns:
+        `None` if cannot be parsed as fraction, `Fraction <fractions.Fraction>`
+        otherwise.
+    '''
+    try:
+        return Fraction(''.join(string.split(sep=None)))
+    except ValueError:
+        return None
+
+
 def validation_pass(label: str, why: typing.Optional[model.VWhy] = None, **kwargs) -> typing.Optional[bool]:
     '''Neutral validation function, accepts everything.
 
@@ -614,3 +671,89 @@ def window_center(window: typing.Union[tk.Tk, tk.Toplevel], where: tk.Tk,
     #     logger.debug('Geometry:: %s', geometry)
     window.geometry(geometry)
     return window.geometry() == geometry
+
+
+if __debug__:
+    def debugWidget(widget):
+        '''Log all information about the given widget.
+
+        As the name implies, this is a debug function, only available on
+        `__debug__`.
+
+        This should be used like this, on the `RootWindow`:
+
+        .. code:: python
+
+            def setup_adefaults(self):
+                if __debug__:
+                    BindingGlobal(self, '<Shift-Button-2>', fn.onDebugWidget,
+                                  immediate=True, description='Debug GUI for Current Widget')
+                    BindingGlobal(self, '<Control-Shift-Button-2>', fn.onDebugPWidget,
+                                  immediate=True, description='Debug GUI on Parent Widget')
+
+        Choose your own binding keys, these are just an example.
+
+        See Also:
+            See the wrapper functions `onDebugWidget`, `onDebugPWidget`.
+
+            See the :envvar:`TKMILAN_DEBUG` environment variable for extra
+            debug settings.
+        '''
+        assert __debug__
+        import os
+        from . import mixin
+
+        DEBUG_LIST = os.environ.get('TKMILAN_DEBUG', '').split(',')
+
+        logger.debug('%s [%s]', widget, widget.__class__.__qualname__)
+        if widget is None:
+            return
+        if not isinstance(widget, mixin.MixinWidget):
+            # Not a "proper" widget, some internal helper
+            pass
+        else:
+            logger.debug('| %s', widget.gstate)
+            if widget.gstate.__class__ != model.GuiState:
+                logger.debug('| - Real: %s', widget.gstate.real())
+            if hasattr(widget, 'wlabel'):
+                logger.debug('| Label: %s', widget.wlabel)
+            wgrid = widget.wgrid
+            if wgrid:
+                logger.debug('| Grid: %s', wgrid)
+                if (wparent := widget.wparent) is not None:
+                    logger.debug('| | r=%r', wparent.rowconfigure(wgrid.row))
+                    logger.debug('| | c=%r', wparent.columnconfigure(wgrid.column))
+            if isinstance(widget, mixin.ContainerWidget):
+                logger.debug('| Widgets:')
+                for wname, w in widget.widgets.items():
+                    logger.debug('| | %s: %s Grid(%s)', wname, w.gstate, w.wgrid)
+                gsize = widget.gsize
+                wcols, wrows = widget.grid_size()
+                logger.debug('| %r widget=%dR%dC', gsize, wrows, wcols)
+                if 'grid' in DEBUG_LIST:
+                    if gsize is not None:
+                        for r in range(gsize.rows):
+                            logger.debug('| | R#%d: %r', r, widget.rowconfigure(r))
+                        for c in range(gsize.columns):
+                            logger.debug('| | C#%d: %r', c, widget.columnconfigure(c))
+                if widget != widget.wroot:
+                    wstate_lines = pformat(widget.wstate).splitlines()
+                    logger.debug('State: %s', wstate_lines[0])
+                    for ln in wstate_lines[1:]:
+                        logger.debug('     : %s', ln)
+            elif isinstance(widget, mixin.SingleWidget):
+                logger.debug('| Widget: %r', widget.wstate)
+                logger.debug('| - Variable: %s', widget.variable)
+            else:
+                raise NotImplementedError(f'{widget}: Unknown class: {widget.__class__.__qualname__}')
+
+    def onDebugWidget(event):
+        '''Run `debugWidget` on the event widget.'''
+        return debugWidget(event.widget)
+
+    def onDebugPWidget(event):
+        '''Run `debugWidget` on the event widget's parent widget.'''
+        widget = event.widget
+        if widget.wparent is None:
+            assert str(widget) == '.', f'Invalid widget parent: {widget!r} » {widget.wparent!r}'
+        return debugWidget(widget.wparent)
