@@ -19,12 +19,14 @@ import tkinter as tk
 ProxyWidgetT = typing.TypeVar('ProxyWidgetT', bound='MixinWidget')
 if typing.TYPE_CHECKING:
     from . import mixin
-    from . import RootWindow
+    from . import RootWindow, Tooltip
 
 logger = logging.getLogger(__name__)
 logger_traces = logging.getLogger('%s.traces' % __name__)
 logger_valid = logging.getLogger('%s.valid' % __name__)
 logger_grid = logging.getLogger('%s.grid' % __name__)
+if __debug__:
+    logger_autolayout = logging.getLogger('%s.autolayout' % __name__)
 
 WEIRD_WIDGET_NAME = (  # Weird Widget `dir` names, these cause trouble
     '_last_child_ids',
@@ -179,9 +181,13 @@ class MixinStateContainer(MixinState):
         dictionary mapping subwidget identifier to `WidgetDynamicState`.
     '''
     wstate_static: bool = False
-    wstate_single: typing.Optional[str] = None
+    wstate_single: typing.Union[None, bool, str] = None
     '''
     Mark the container state as "single", including only the state for this child.
+    The value should be a string indicating the state key.
+
+    `True` means the only existing key should be considered. This should be
+    used only when the key name is dynamic and cannot be statically determined.
 
     Should only be enabled where there is a single child element, this is
     verified when getting the value. Use `MixinWidget.ignoreContainerState` to
@@ -189,9 +195,16 @@ class MixinStateContainer(MixinState):
 
     This creates no performance improvements, it is only useful to simplify the
     state.
+
+    .. note::
+
+        To skip the warning about a container with a single widget on debug
+        binaries, mark the widget class with ``wstate_single = False``.
+        This works exactly as the default `None`, without the warning.
     '''
 
-    def state_get(self, **kwargs):
+    def state_get(self, **kwargs) -> typing.Any:
+        assert self.wstate_single != '', f'{self}: Invalid "wstate_single" marking, cannot be an empty string'
         state = {}
         for identifier, wds in self.stateSetup(**kwargs).items():
             result = wds.getter()
@@ -199,23 +212,36 @@ class MixinStateContainer(MixinState):
                 pass  # Skip
             else:
                 state[identifier] = result
-        if self.wstate_single is None:
+        if self.wstate_single is None or self.wstate_single is False:
             # - Multiple WState
             if len(state) == 0:
                 return None
             else:
                 if __debug__:
                     wid0 = list(state)[0]
-                    if len(state) == 1 and ':' not in wid0:
+                    # Do not warn if:
+                    # - `:` in widget key
+                    #   - Internal Widgets
+                    # - `wstate_single is False`
+                    #   - Manual Override, `None` is the default
+                    if len(state) == 1 and ':' not in wid0 and self.wstate_single is not False:
                         # TODO: Use `warnings.warn`, but this is used from an `after` function, the trace is lost.
                         logger.warning('%s: This widget can be marked "wstate_single": `%s`', self, wid0)
                 return state
         else:
             # - Single WState (wstate_single)
             assert len(state) == 1, f'{self}: Invalid "wstate_single" marking, {len(state)} elements'
-            return state[self.wstate_single]
+            single_key: str
+            if self.wstate_single is True:
+                single_key = list(state)[0]
+            else:
+                assert isinstance(self.wstate_single, str)
+                single_key = self.wstate_single
+            assert single_key in state, f'{self}: Invalid "wstate_single" marking, missing key'
+            return state[single_key]
 
     def state_set(self, state, substate, **kwargs) -> None:
+        assert self.wstate_single != '', f'{self}: Invalid "wstate_single" marking, cannot be an empty string'
         # # Debug container state flow
         # self_names = None
         # if __debug__:
@@ -231,14 +257,16 @@ class MixinStateContainer(MixinState):
             if wds.noneable and state is None:
                 skip = True
             else:
-                if self.wstate_single is None:
+                if self.wstate_single is None or self.wstate_single is False:
                     # - Multiple WState
                     #   - Check for "noneable"
                     #   - Check for (substate=True)
                     skip = (substate or wds.noneable) and identifier not in state
                 else:
                     # - Single WState (wstate_single)
-                    skip = self.wstate_single != identifier
+                    #   - Skip all but the given key
+                    #   - Do not skip when True
+                    skip = self.wstate_single is not True and self.wstate_single != identifier
             if skip:
                 # if __debug__:
                 #     logger.debug('%s|> Skip "%s"', ' ' * len(self_names), identifier)
@@ -247,8 +275,10 @@ class MixinStateContainer(MixinState):
                 # if __debug__:
                 #     logger.debug('%s|>  Set "%s"', ' ' * len(self_names), identifier)
                 if self.wstate_single:
+                    assert (self.wstate_single is True) or isinstance(self.wstate_single, str)
                     widget_state = state
                 else:
+                    assert self.wstate_single in (None, False)
                     widget_state = state[identifier]
                 if wds.container:
                     wds.setter(widget_state, substate, **kwargs)
@@ -455,6 +485,7 @@ class MixinWidget:
         if isinstance(fmt, str):
             fmt = model.GridCoordinates.parse(fmt)
         kwargs.update(fmt.dict())
+        assert self.ignoreContainerLayout is False, f'{self}: Layout is being ignored'
         self.grid(**kwargs)
 
     @property
@@ -818,7 +849,15 @@ class MixinValidationSingle:
         assert self.vsettings.fn is not None
         assert self.vsettings.fnSimple is True
         rval = self.vsettings.fn(always_vstate, None)
-        logger_valid.debug('%s: validation=%s', self, rval)
+        if self.vsettings.tt is not None and self.vsettings.ttSimple is True:
+            self.__validTooltipSimple(rval is True)
+            if __debug__:
+                tt_log = f' tt:simple:{self.vsettings.tt}'
+        else:
+            if __debug__:
+                tt_log = ''
+        if __debug__:
+            logger_valid.debug('%s: validation=%s%s', self, rval, tt_log)
         return 'break' if rval is None else rval
 
     def __validCommandComplex(self,
@@ -838,9 +877,22 @@ class MixinValidationSingle:
         assert self.vsettings is not None
         assert self.vsettings.fn is not None
         assert self.vsettings.fnSimple is False
-        vwhy = model.VWhy(vstate=always_vstate, widget=self, why=why, t=whyNum)
+        if self.vsettings.tt is not None and self.vsettings.ttSimple is True:
+            ttobj = None
+        else:
+            ttobj = self.vsettings.tt
+        vwhy = model.VWhy(vstate=always_vstate, why=why, t=whyNum,
+                          widget=self, tt=ttobj)
         rval = self.vsettings.fn(always_vstate, vwhy)
-        logger_valid.debug('%s: validation=%s why=%s', self, rval, vwhy)
+        if self.vsettings.tt is not None and self.vsettings.ttSimple is True:
+            self.__validTooltipSimple(rval is True)
+            if __debug__:
+                tt_log = ' tt:simple'
+        else:
+            if __debug__:
+                tt_log = ''
+        if __debug__:
+            logger_valid.debug('%s: validation=%s%s why=%s', self, rval, tt_log, vwhy)
         return 'break' if rval is None else rval
 
     def __validPost(self, var, etype, *, vtype: model.ValidateWhyT):
@@ -848,6 +900,14 @@ class MixinValidationSingle:
         # Keep the GUI "valid" state up to date
         self.set_gui_state(valid=self.__validCommand(var.get(), vtype, -1))
 
+    def __validTooltipSimple(self, rval: typing.Optional[bool]):
+        assert self.vsettings is not None
+        assert self.vsettings.tt is not None
+        if rval is True:
+            self.vsettings.tt.disable()
+        else:
+            self.vsettings.tt.enable()
+
     def setDefault(self) -> None:
         '''Set the current state to the default label.'''
         assert isinstance(self, SingleWidget), f'{self.__class__.__qualname__}: Unsupported validation for this Widget'
@@ -879,6 +939,64 @@ class MixinValidationSingle:
         assert isinstance(self.variable, tkmilan_var.SpecParsed), f'{self.__class__.__qualname__}: Non-`SpecParsed` variable type'
         return self.eSet(self.variable.getLabel(value))
 
+    # TODO: On Python 3.11:: -> typing.Self
+    def putTooltip(self, tooltipClass: 'typing.Union[bool, typing.Type[Tooltip]]' = True,
+                   **tt_kwargs,
+                   ) -> 'MixinValidationSingle':
+        '''Setup the tooltip object for the validation settings on itself.
+
+        This creates the `Tooltip` object according to the settings, attached
+        to the current widget. See `VSettings <model.VSettings>` for tooltip
+        related settings.
+        The widget must setup the validation already, but **must not** define
+        an existing tooltip object (parameter ``tt``).
+
+        This is designed to be used inside the `setup_widgets
+        <ContainerWidget.setup_widgets>` function, like this:
+
+        .. code:: python
+
+            def setup_widgets(self, ...):
+                self.w1 = Widget(self, ...).putTooltip(...)
+
+        This is usually called "method chaining", or "fluent interface".
+
+        Args:
+            tooltipClass: The class to instance as the tooltip object.
+                Must be a `Tooltip`, or `True` to use `TooltipValidation`.
+
+                Defaults to `True`.
+
+        All other keyword arguments are passed to the `Tooltip` creation.
+        '''
+        from . import TooltipValidation  # Avoid circular import
+        assert isinstance(self, MixinWidget), f'{self} is not a valid tkmilan widget'
+        if self.vsettings is None:
+            raise ValueError(f'{self}: Widget validation not configured')
+        assert self.vsettings is not None
+        if tooltipClass is False:
+            if __debug__:
+                warnings.warn('{self}: `.putTooltip(False)` is pointless, remove', stacklevel=2)
+        else:
+            tt_obj: Tooltip
+            if tooltipClass is True:
+                tt_obj = TooltipValidation(self, **tt_kwargs)
+            else:
+                tt_obj = tooltipClass(self, **tt_kwargs)
+            if __debug__:
+                logger_valid.debug('%s: tooltip%s=%s ttSimple=%s', self,
+                                   '[V]' if isinstance(tt_obj, TooltipValidation) else '', tt_obj,
+                                   self.vsettings.ttSimple)
+            if isinstance(tt_obj, TooltipValidation):
+                if isinstance(self, SingleWidget):
+                    vobj = self.variable
+                else:  # TODO: Can be used on other types of widgets?
+                    vobj = None
+                tt_obj.setup_tooltip_state(vobj)
+            assert self.vsettings.tt is None, f'{self}: Overwriting validation tooltip is unsupported'
+            self.vsettings.tt = tt_obj
+        return self
+
 
 # High-Level Mixins
 
@@ -887,6 +1005,11 @@ class SingleWidget(MixinWidget, MixinStateSingle, MixinTraces):
     '''Parent class of all single widgets.'''
     variable: typing.Optional[tkmilan_var.Variable] = None
     state_type: typing.Optional[typing.Type[tkmilan_var.Variable]] = None
+    layout_padable: bool = True
+    '''Should this widget be automatically padded, when requested.
+
+    See `ContainerWidget.pad_container`.
+    '''
 
     def init_single(self,
                     vspec: typing.Union[None, tkmilan_var.Variable, validation.VSpec] = None,
@@ -998,6 +1121,8 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
 
     Most containers behave well when padding children, but some are
     problematic, mark those at class level (set ``layout_padable = False``).
+
+    See `ContainerWidget.pad_container`.
     '''
     variable: typing.Optional[tkmilan_var.aggregator] = None
 
@@ -1244,51 +1369,72 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
     def var(self, cls: typing.Type[tkmilan_var.Variable], *,
             value: typing.Optional[typing.Any] = None,
             name: typing.Optional[str] = None,
+            vname: typing.Optional[str] = None,
             ) -> 'tkmilan_var.Variable':
         '''"Attach" a new variable to this container.
 
         Args:
             cls: The variable class.
 
-            name: The variable name. Optional, defaults to an autogenerated name.
+            name: The variable name, on the widget namespace.
+                Optional, defaults to ``vname``.
             value: The default value. Optional, defaults to `None`.
+            vname: The variable name, on the global namespace.
+                Optional, defaults to an autogenerated name.
 
         See Also:
             - `varSpec`: Attach a `Spec` variable to this container, with a name.
             - `gvar`: Access the variable by name.
+
+        .. note::
+
+            The ``vname`` is defined in a global namespace, common to the
+            entire application. This can be used to define common variables
+            across different widgets, but this is not default behaviour since
+            it violates the principle of least surprise.
         '''
-        vobj = cls(name=name)
+        vobj = cls(name=vname)
         if value is not None:
             vobj.set(value)
         assert isinstance(vobj, tkmilan_var.Variable), f'Class "{cls}" is not a "Variable"'
         # Save the variables on the instance object
-        vname = name or str(vobj)
-        self._variables[vname] = vobj
+        kname = name or vname or str(vobj)
+        self._variables[kname] = vobj
         return vobj
 
     def varSpec(self, cls: typing.Type[tkmilan_var.Spec], spec: typing.Any, *,
                 name: typing.Optional[str] = None,
+                vname: typing.Optional[str] = None,
                 ) -> 'tkmilan_var.Spec':
         '''"Attach" a new `specified variable <var.Spec>` to this container.
 
         Args:
             cls: The variable class.
-            name: The variable name. Optional, defaults to an autogenerated name.
-
-            All other arguments are passed to the class constructor.
+            name: The variable name, on the widget namespace.
+                Optional, defaults to ``vname``.
+            spec: The specification to creating the new variable.
+            vname: The variable name, on the global namespace.
+                Optional, defaults to an autogenerated name.
 
         See Also:
             - `var`: Attach a non-`Spec` variable to this container, with a name.
             - `gvar`: Access the variable by name.
+
+        .. note::
+
+            The ``vname`` is defined in a global namespace, common to the
+            entire application. This can be used to define common variables
+            across different widgets, but this is not default behaviour since
+            it violates the principle of least surprise.
         '''
         if __debug__:
             if cls in tkmilan_var.Limit.__subclasses__():
                 warnings.warn('No need to use `var.Limit` subclasses, use the parent class', stacklevel=2)
-        vobj = cls(spec, name=name)
+        vobj = cls(spec, name=vname)
         assert isinstance(vobj, tkmilan_var.Spec), f'Class "{cls}" is not a "Spec"'
         # Save the variables on the instance object
-        vname = name or str(vobj)
-        self._variables[vname] = vobj
+        kname = name or vname or str(vobj)
+        self._variables[kname] = vobj
         return vobj
 
     def gvar(self, name: str) -> tkmilan_var.Variable:
@@ -1307,11 +1453,13 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
 
     def layout_container(self, layout: typing.Optional[str], widgets: typing.Sequence[MixinWidget]):
         assert isinstance(self, (tk.Widget, tk.Tk, tk.Toplevel)), f'{self} is not a valid widget'
-        if self.layout_expand:
+        if not self.ignoreContainerLayout and self.layout_expand:
             assert isinstance(self, tk.ttk.Widget), f'{self} is not a valid tkinter.ttk.Widget'
             self.grid(sticky=tk.NSEW)
         # Automatic Layout
         try:
+            if __debug__:
+                logger_autolayout.debug('%s: AutoLayout %d widgets', self, len(widgets))
             layout, args = autolayout.do(layout, len(widgets))
         except Exception as e:
             logger.critical('%s: Weird Layout: %s [%s]', self, layout, e)
@@ -1338,6 +1486,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
             for idx, (arg, widget) in enumerate(zip(args, widgets)):
                 widget_real = widget.wproxy or widget
                 assert isinstance(widget_real, (tk.Widget, tk.Tk, tk.Toplevel)), f'Widget #{idx}[{str(widget_real)}]: Type {type(widget_real)}'
+                assert widget.ignoreContainerLayout is False and widget_real.ignoreContainerLayout is False, f'{self}: Layout is being ignored'
                 widget_real.grid(**arg.dict())  # Change the grid on the proxy widget
                 if self.layout_autoadjust:
                     # Check the inner widget, no proxies
@@ -1364,6 +1513,11 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
                     self.pgrid_c(*_separator_ws,
                                  weight=0)
         self.setup_layout(layout)  # Custom adjustments, after all automatic changes
+        if __debug__:
+            # Check the container layout is sane
+            wcols, wrows = self.grid_size()
+            wgsize = model.GridSize(rows=wrows, columns=wcols)
+            assert self.gsize == wgsize, f'{self}: Invalid Container Grid: f={self.gsize} w={wgsize}'
 
     def pad_container(self, pad: int, *, recursive: bool = True, _level: int = 0) -> None:
         '''Pad this container widget, recursively.
@@ -1396,9 +1550,10 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
         pad_container = len(self.widgets) > 1 and self.layout_padable
         for w in self.widgets.values():
             realw = w.wproxy or w
-            if pad_container and not w.ignoreContainerLayout:
+            assert isinstance(realw, (SingleWidget, ContainerWidget))
+            if pad_container and not w.ignoreContainerLayout and realw.layout_padable:
                 assert isinstance(realw, tk.Widget), f'{w}: Invalid widget for padding'
-                ginfo = realw.grid_info()
+                ginfo = realw.grid_info()  # TODO: Use `realw.wgrid`
                 if ginfo.get('column', 0) == 0:  # First Column Index
                     # Left Widget
                     # - Pad Both sides
@@ -1427,8 +1582,17 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
 
     @property
     def gsize(self) -> model.GridSize:
-        '''GUI grid size (according to the current child widgets).'''
-        return fn.grid_size(*[w.wproxy or w for w in self.widgets.values()])  # Use the proxy widget
+        '''GUI grid size (according to the current child widgets).
+
+        Follow `MixinWidget.ignoreContainerLayout` setting.
+        '''
+        # Different from `tkinter.Widget.grid_size`, might be temporarily broken
+        # befor the whole layout setup runs.
+        return fn.grid_size(*[
+            w.wproxy or w  # Use the proxy widget
+            for w in self.widgets.values()
+            if w.ignoreContainerLayout is False
+        ])
 
     def state_c(self, *, vid_upstream: typing.Optional[typing.Set[str]] = None) -> ContainerState:
         swidgets = {}
@@ -1714,6 +1878,8 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
         Filter all child widgets by type. Very useful to apply settings to a
         subset of all child widgets.
 
+        Consider proxy widgets, do the right thing.
+
         Arguments:
             classes: Widget Types, as classes.
 
@@ -1723,7 +1889,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
         class_tuple = tuple(classes)
         for w in self.widgets.values():
             if isinstance(w, class_tuple):
-                yield w
+                yield w.wproxy or w
 
     def widgets_except(self, *widgets: MixinWidget) -> typing.Iterable[MixinWidget]:
         '''Filter widgets reversed by instance.
@@ -1732,6 +1898,8 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
         other widgets.
         Very useful to apply settings to a subset of all child widgets.
 
+        Consider proxy widgets, do the right thing.
+
         Arguments:
             widgets: Child widgets to skip.
 
@@ -1739,5 +1907,5 @@ class ContainerWidget(MixinWidget, MixinStateContainer, MixinTraces):
             See `widgets_class` to filter child widgets by class.
         '''
         for w in self.widgets.values():
-            if w not in widgets:
-                yield w
+            if w not in widgets or w.wproxy in widgets:
+                yield w.wproxy or w
