@@ -23,7 +23,7 @@ if typing.TYPE_CHECKING:
     # Avoid circular imports
     from . import mixin
     from . import var
-    from . import EntryMultiline
+    from . import EntryMultiline, Tooltip
 
 ImageTk: 'typing.Optional[types.ModuleType]'  # Optional Dependency: Pillow
 try:
@@ -74,6 +74,25 @@ Their meanings are as follows:
 There is no Python documentation, see ``Tk`` :tk:`trace validation action type
 <ttk_entry.html#M43>` documentation.
 '''
+NotebookTabStateT = typing.Literal[
+    # Shown
+    'normal', 'disabled',
+    # Hidden
+    'hidden',
+]
+'''The supported `NotebookTab` state types.
+
+Their meanings are as follows:
+
+- ``normal``: A regular tab
+- ``disabled``: The tab is shown, but disabled. This means it cannot be
+   selected, not even programatically.
+- ``hidden``: The tab is not shown. If you show it (using `Notebook.wselect`),
+   it goes back to ``normal`` state.
+
+There is no Python documentation, see ``Tk`` :tk:`notebook tab state type
+<ttk_notebook.html#M11>` documentation.
+'''
 
 
 # Image Types
@@ -267,7 +286,15 @@ class SStyle:
 
     Works for `FrameLabelled` and friends: `FrameStateful`, `FrameRadio`, etc.
     '''
-    # TODO: Store `place` arguments, keyed by "anchor": CP.
+    # TODO: `Size_YF_Frame`:: Store `place` arguments, keyed by "anchor": CP.
+    Size_PadWidget_FrameSeparator: int = 8
+    '''Size: ``padding`` with the same size as a frame separator (e.g.
+    `FrameLabelled`, `FrameStateful`, `FrameRadio`).
+
+    When stacking a `FrameRadio` and a `Radio` button vertically, pad the
+    `Radio` widget on the left side and align it to the left, to obtain aligned
+    `Radio` widgets.
+    '''
 
     def __init__(self):
         raise ValueError('Not to be Instanced')
@@ -295,6 +322,8 @@ class DStyle(enum.Enum):
     See Also:
         `SStyle` for static style values.
     '''
+    Color_FG = enum.auto()
+    '''Color: Normal Widgets ``foreground``'''
     Color_BG = enum.auto()
     '''Color: Normal Widgets ``background``'''
     Color_FG_Disabled = enum.auto()
@@ -304,6 +333,10 @@ class DStyle(enum.Enum):
     Font_Default = enum.auto()
     '''Font: Default Font'''
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
 
 class CP(enum.Enum):
     '''A Cardinal Point.
@@ -336,6 +369,10 @@ class CP(enum.Enum):
     default = None
     '''OS-specific cardinal point.'''
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
 
 CP_Compound: typing.Mapping[compoundT, str] = {
     CP.default: 'none',  # `image` if present, otherwise `text`
@@ -383,6 +420,10 @@ class Justification(enum.Enum):
     Right = tk.RIGHT
     '''Justify Right'''
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
 
 Justification_CP: typing.Mapping[Justification, CP] = {
     Justification.NoJustify: CP.default,
@@ -616,6 +657,10 @@ class Direction(enum.Enum):
         self.dC = dC
         assert len(self.name) == 1, 'Direction name must be a single character'
 
+    if __debug__:
+        def __repr__(self) -> str:
+            return f'{self.__class__.__qualname__}.{self.name}'
+
     def grid(self, rows: int, cols: int, amount: typing.Optional[int] = None, auto_fill: bool = True) -> typing.Iterable[GridCoordinates]:
         '''Generate `GridCoordinates` for a widget grid.
 
@@ -835,14 +880,23 @@ class VWhy:
 
     Args:
         vstate: The widget validated state.
-        why: Validation script substitution: validation condition (``%V``). See `ValidateWhyT`.
-        t: Validation script substitution: action type (``%d``). See `ValidateST`.
+        why: Validation script substitution: validation condition (``%V``).
+
+            See `ValidateWhyT`.
+        t: Validation script substitution: action type (``%d``).
+
+            See `ValidateST`.
         widget: The widget being validated.
+        tt: The widget tooltip. Optional.
+
+            When this is `None`, it means the automatic tooltip handling will
+            run, and *vice versa*.
     '''
     vstate: VState
     why: ValidateWhyT  # %V
     t: ValidateST      # %d
     widget: 'mixin.MixinWidget'
+    tt: 'typing.Optional[Tooltip]'
 
     def validation(self) -> typing.Optional[bool]:
         '''Run the natural validation for this reason.
@@ -890,6 +944,16 @@ class VSettings:
         fnSimple: Do not calculate the full validation data, send only the
             widget state. This is usually the only thing that matters.
             Defaults to enabled, for performance reasons.
+        tt: The corresponding `Tooltip` for this widget, if available.
+            This is paired with the `putTooltip
+            <mixin.MixinValidationSingle.putTooltip>` function.
+            Optional, not to be used directly.
+        ttSimple: When `True`, setup a very simple tooltip behaviour: disable
+            when valid, enable otherwise. Set this to `False` for more complex
+            behaviours.
+            Defaults to `None`, meaning dependent on other options. When ``fn``
+            is `None` or ``fnSimple`` is `False`, default to `True`, with this
+            the user has no way to control this. Otherwise, default to `False`.
 
         tkWhen: Calculated ``Tk`` setting, based on the other parameters. See
             :tk:`validation modes <ttk_entry.html#M35>` documentation.
@@ -904,10 +968,14 @@ class VSettings:
     # Function
     fn: typing.Optional[typing.Callable[[VState, typing.Optional[VWhy]], typing.Optional[bool]]] = None
     fnSimple: bool = True
+    # Tooltip (Optional)
+    tt: 'typing.Optional[Tooltip]' = dc_field(init=False, default=None)
+    ttSimple: typing.Optional[bool] = None  # Never `None`
     # tkinter values
     tkWhen: ValidateWhenT = dc_field(init=False)
 
     def __post_init__(self, postFocusIn: bool, postFocusOut: bool, preKey: bool):
+        # tkWhen
         tkWhen: typing.Optional[str] = {
             (True, True, True): 'all',
             (True, False, False): 'focusin',
@@ -920,6 +988,16 @@ class VSettings:
         assert tkWhen in ('focus', 'focusin', 'focusout', 'key', 'all', 'none')
         tkWhen = typing.cast(ValidateWhenT, tkWhen)  # All good
         self.tkWhen = tkWhen
+        # tt
+        # - Never check `self.tt` here, it's usually defined later
+        if self.ttSimple is None:
+            self.ttSimple = self.fn is None or self.fnSimple is True
+            # if __debug__:
+            #     logger.debug('%s: Default ttSimple=%s', self, self.ttSimple)
+        else:
+            if self.fnSimple is True:
+                assert self.ttSimple is True, f'Invalid combination (unreachable tooltip): fnSimple={self.fnSimple} ttSimple={self.ttSimple}'
+        assert isinstance(self.ttSimple, bool), f'Bad ttSimple: {self.ttSimple!r}'
 
 
 @dataclass
@@ -1043,6 +1121,18 @@ class GuiState:
         '''
         return cls(**dict(state.items()))
 
+    if __debug__:
+        # Include a nicer representation
+        # - Skip `None` values
+        # - If printed with fixed width fonts, they align vertically
+        def __repr__(self) -> str:
+            str_elements = ', '.join(
+                f'{estr}={str(sval)[0]}'
+                for estr, sval in self.items()
+                if sval is not None
+            )
+            return f'{self.__class__.__qualname__}({str_elements})'
+
 
 @dataclass
 class GuiState_Tk:
@@ -1147,15 +1237,44 @@ class MixinBinding:
     def _unbind(self, obj: typing.Any) -> None:
         raise NotImplementedError
 
-    def enable(self) -> None:
-        '''Enable the binding.'''
+    def enable(self, *, warn: bool = True) -> None:
+        '''Enable the binding.
+
+        Args:
+            warn: Warn about redundant operations.
+                Enabled by default.
+
+        See Also:
+            Use `tryenable` to force ``warn`` to `False`.
+        '''
         if self.obj is None:
             self.obj = self._bind()
         else:
-            warnings.warn(f'Redundant enable @ {self!r}')
+            if warn:
+                warnings.warn(f'Redundant enable @ {self!r}', stacklevel=2)
 
-    def disable(self) -> None:
-        '''Disable the binding.'''
+    def tryenable(self) -> bool:
+        '''Try to enable the binding, don't warn when already enabled.
+
+        Wraps `enable`, with ``warn`` forced to `False`.
+
+        Returns:
+            A `bool` indicating if the state has changed.
+        '''
+        ostate = bool(self)
+        self.enable(warn=False)
+        return ostate != bool(self)
+
+    def disable(self, *, warn: bool = True) -> None:
+        '''Disable the binding.
+
+        Args:
+            warn: Warn about redundant operations.
+                Enabled by default.
+
+        See Also:
+            Use `trydisable` to force ``warn`` to `False`.
+        '''
         if self.obj is not None:
             obj = self.obj
             try:
@@ -1165,7 +1284,20 @@ class MixinBinding:
                 self.obj = obj  # Restore
                 raise Exception(f'Error when unbinding @ {self.widget!r}') from e
         else:
-            warnings.warn(f'Redundant disable @ {self!r}')
+            if warn:
+                warnings.warn(f'Redundant disable @ {self!r}', stacklevel=2)
+
+    def trydisable(self) -> bool:
+        '''Try to disable the binding, don't warn when already disabled.
+
+        Wraps `disable`, with ``warn`` forced to `False`.
+
+        Returns:
+            A `bool` indicating if the state has changed.
+        '''
+        ostate = bool(self)
+        self.disable(warn=False)
+        return ostate != bool(self)
 
 
 class Binding(MixinBinding):
@@ -1202,7 +1334,9 @@ class Binding(MixinBinding):
 
     .. automethod:: __bool__
     .. automethod:: enable
+    .. automethod:: tryenable
     .. automethod:: disable
+    .. automethod:: trydisable
     '''
     def __init__(self, widget: 'mixin.MixinWidget', sequence: str, *args, **kwargs):
         if sequence in widget.wroot._bindings_global:
@@ -1252,7 +1386,9 @@ class BindingGlobal(MixinBinding):
 
     .. automethod:: __bool__
     .. automethod:: enable
+    .. automethod:: tryenable
     .. automethod:: disable
+    .. automethod:: trydisable
     '''
     def __init__(self, widget: 'mixin.MixinWidget', sequence: str, *args, **kwargs):
         root_widget = widget.wroot
@@ -1296,7 +1432,9 @@ class BindingTag(MixinBinding):
 
     .. automethod:: __bool__
     .. automethod:: enable
+    .. automethod:: tryenable
     .. automethod:: disable
+    .. automethod:: trydisable
     '''
     def __init__(self, widget: 'EntryMultiline', tag: str, sequence: str, *args, **kwargs):
         self.tag = tag
@@ -1900,6 +2038,69 @@ class NotebookTab:
         else:
             self.imageCompound = CP_Compound[False]
 
+    @property
+    def tk_idx(self) -> str:
+        '''Get the ``Tk`` index to be used on upstream functions as needed.
+
+        This uniquely identifies the correct tab.
+        '''
+        return str(self.widget.wproxy or self.widget)
+
+
+@dataclass(init=False, frozen=True)
+class NotebookTabOrder:
+    '''Information for all `Notebook <tkmilan.Notebook>` tab ordering.
+
+    These are the parameters which can be configured:
+
+    Args:
+        shown: Tab identiers to be shown, either on normal and disable modes.
+            **Mandatory**.
+        disabled: Tab identifiers to be shown as disabled.
+            Optional, defaults to an empy set.
+
+    Other parameters are automatically calculated:
+
+        active: Tab identifiers to be show as active.
+    '''
+    shown: typing.Sequence[str]
+    disabled: typing.Set[str]
+    active: typing.Sequence[str] = dc_field(repr=False, compare=False)
+
+    def __init__(self, shown: typing.Iterable[str], *,
+                 disabled: typing.Optional[typing.Iterable[str]] = None):
+        object.__setattr__(self, 'shown', tuple(shown))
+        object.__setattr__(self, 'disabled', set(disabled or ()))
+        if __debug__:
+            active_set = tuple(sorted(set(self.shown)))
+            active_sort = tuple(sorted(self.shown))
+            delta_active = set(active_sort) - set(active_set)
+            assert active_set == active_sort, f'Repeated shown keys: `{" ".join(delta_active)}`'
+            bad_disabled = self.disabled - set(self.shown)
+            assert len(bad_disabled) == 0, f'Invalid disabled keys: `{" ".join(bad_disabled)}`'
+        object.__setattr__(self, 'active', tuple(e for e in shown if e not in self.disabled))
+
+    def tk_states(self, allidx: typing.Iterable[str],
+                  ) -> typing.Iterator[typing.Tuple[str, typing.Optional[int], NotebookTabStateT]]:
+        '''Generate ``Tk`` information about all `NotebookTab`.
+
+        Given all the possible tab identifiers, generate tuples with the following information:
+
+        - ``idx``: The `NotebookTab` identifier
+        - ``index``: The index to move the `NotebookTab` to. `None` if the tab
+          should be hidden.
+        - ``state``: The `NotebookTab` state to be configured. See
+          `NotebookTabStateT`.
+
+        Args:
+            allidx: All `NotebookTab` identifiers.
+        '''
+        for index, idx in enumerate(self.shown):
+            yield idx, index, tk.DISABLED if idx in self.disabled else tk.NORMAL
+        for idx in set(allidx) - set(self.shown):
+            # `Normal?` is not used
+            yield idx, None, tk.HIDDEN
+
 
 @dataclass
 class TreeColumn:
