@@ -6,6 +6,7 @@ import typing
 from dataclasses import dataclass, field as dataclass_field
 from collections import Counter
 from html.parser import HTMLParser
+from html import escape as html__escape
 
 from . import model
 
@@ -92,7 +93,7 @@ class LTML(HTMLParser):
     Args:
         tag_simplify: Simplify some tags (`TAGS_SIMPLE`).
         tag_act: See the description for automatic counter tags
-        documentation. Defaults to `True`.
+            documentation. Defaults to `True`.
 
     .. automethod:: feed
 
@@ -256,6 +257,22 @@ class LTML(HTMLParser):
         return self.cmdlist
 
 
+def escape_LTML(string: str, *, quote: bool = True) -> str:
+    '''Convert the ``string`` into safe Lite Text Markup Language.
+
+    Wraps the upstream escape function, see `html.escape`.
+
+    .. note:: You can use the upstream function directly, instead of importing
+        this.
+
+    Arguments:
+        string: String to escape special LTML characters.
+        quote: Escape quotes too. Usually needed, since it might be embedded in
+            an attribute. Defaults to `True`.
+    '''
+    return html__escape(string, quote=quote)
+
+
 def parse_LTML(ltml: str, __parser: LTML = LTML()) -> typing.Sequence[model.TextElement]:
     '''Parse Lite Text Markup Language using the default settings.
 
