@@ -158,14 +158,14 @@ def StaticMapValues(fn: typing.Callable[[smvT], str], lst: typing.Sequence[smvT]
 
 
 @dataclass
-class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
-    '''Represent a bounded range to limit a value, generic for any `number
-    <numbers.Number>` type.
+class LimitBounded(typing.Mapping[str, int], VSpec):
+    '''Represent a bounded range to limit a value, for integers (`int`).
 
     The range can be made to remove the limits themselves with ``imin`` and
     ``imax``. The default is the limit being closed on both sides.
 
-    The range is bounded, there are no infinite limits.
+    The range is bounded, there are no infinite limits. The ``step`` parameter
+    controls if all intermediate values are valid.
 
     The following arguments are not available on the final object:
 
@@ -184,6 +184,8 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
     Arguments:
         fn: A function that turns a string into `number <numbers.Number>`.
             Usually some kind of parser.
+        step: The step amount for the range of values.
+            Defaults to ``1``.
         imin: Include the limit on the minimum value itself.
             Defaults to `True`.
         imax: Include the limit on the maximum value itself.
@@ -199,36 +201,42 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
         rrange: The `range` object implementing this object.
         ldefault: The default label to be shown to the user.
 
+
+    See Also:
+        Use `LimitUnbounded` if the limit is unbounded in any direction, or if
+        the `number <numbers.Number>` is not an `int` subclass.
+
     .. automethod:: __len__
     .. automethod:: __getitem__
     .. automethod:: __iter__
     .. automethod:: __contains__
     '''
-    min_entry: InitVar[typing.Union[limT, str]]
-    max_entry: InitVar[typing.Union[limT, str]]
-    fn: typing.Callable[[str], typing.Optional[limT]]
+    min_entry: InitVar[typing.Union[int, str]]
+    max_entry: InitVar[typing.Union[int, str]]
+    fn: typing.Callable[[str], typing.Optional[int]]
+    step: int = 1
     imin: bool = True
     imax: bool = True
     # Calculations
-    min: limT = dc_field(init=False)
-    max: limT = dc_field(init=False)
+    min: int = dc_field(init=False)
+    max: int = dc_field(init=False)
     min_value: str = dc_field(init=False)
     max_value: str = dc_field(init=False)
-    rrange: typing.Tuple[typing.SupportsIndex, typing.SupportsIndex, typing.SupportsIndex] = dc_field(init=False)
-    # step: typing.Optional[limT] = None  # TODO: Support
+    rrange: range = dc_field(init=False)
     ldefault: str = dc_field(init=False)
     # __init__ variable
-    default: InitVar[typing.Union[limT, str, None]] = None
+    default: InitVar[typing.Union[int, str, None]] = None
 
-    def __post_init__(self, min_entry: typing.Union[limT, str], max_entry: typing.Union[limT, str],
-                      default: typing.Union[limT, str, None] = None):
+    def __post_init__(self, min_entry: typing.Union[int, str], max_entry: typing.Union[int, str],
+                      default: typing.Union[int, str, None] = None):
         if isinstance(min_entry, str):
             self.min_value = min_entry
             min_obj = self.fn(self.min_value)
             if min_obj is None:
                 raise ValueError(f'Invalid Minimum: {min_entry!r}')
             self.min = min_obj
-        else:  # isinstance(min_entry, limT)
+        else:
+            assert isinstance(min_entry, int)
             self.min = min_entry
             self.min_value = str(self.min)
         if isinstance(max_entry, str):
@@ -237,21 +245,35 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
             if max_obj is None:
                 raise ValueError(f'Invalid Maximum: {max_entry!r}')
             self.max = max_obj
-        else:  # isinstance(min_entry, limT)
+        else:
+            assert isinstance(min_entry, int)
             self.max = max_entry
             self.max_value = str(self.max)
         # Range
-        step: limT = typing.cast(limT, 1)  # TODO: Support "step"
-        imin: limT = typing.cast(limT, 0) if self.imin else step
-        imax: limT = step if self.imax else typing.cast(limT, 0)
-        assert isinstance(self.min, int) and isinstance(self.max, int), f'Unsupported Limit type: {self.min!r}/{self.max!r}'
+        assert isinstance(self.step, int), f'Unsupported step: {self.step!r}'
+        if self.step < 1:
+            raise ValueError(f'Invalid Step: {self.step!r}')
+        imin: int = 0 if self.imin else self.step
+        imax: int = 1 if self.imax else 0
+        assert isinstance(self.min, int) and isinstance(self.max, int), f'Unsupported Limit type: {self.min!r}/{self.max!r}; use a `LimitUnbounded`'
         rfrom, rto = self.min + imin, self.max + imax
         assert rto > rfrom
-        if step != typing.cast(Number, 1):
-            # TODO: Support "step"
-            raise ValueError(f'{self}: Unsupported step: {step!r}')
-        self.rrange = range(rfrom, rto, step)
+        self.rrange = range(rfrom, rto, self.step)
         assert len(self.rrange) > 0
+        if self.imax and self.max not in self.rrange:
+            # Bad maximum value, help the user
+            lastvalue = self.rrange[-1]
+            nextvalue = lastvalue + self.step
+            assert lastvalue in self.rrange
+            assert nextvalue not in self.rrange
+            raise ValueError(f'Invalid Step/Maximum: {self.max} not in range; use {lastvalue} or {nextvalue}')
+        elif not self.imax:
+            if self.rrange[-1] + self.step != self.max:
+                nextvalue = self.rrange[-1] + self.step
+                raise ValueError(f'Invalid Step/Maximum: {self.max} is not the correct maximum; use {nextvalue}')
+        if __debug__:
+            assert (self.min in self.rrange) is self.imin
+            assert (self.max in self.rrange) is self.imax
         # Default
         if default is None:
             self.ldefault = {
@@ -262,13 +284,15 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
             }[(self.imin, self.imax)]
         elif isinstance(default, str):
             self.ldefault = default
-        else:  # isinstance(default, limT)
+        else:
+            assert isinstance(default, int)
             self.ldefault = str(default)
             default_roundtrip = self.fn(self.ldefault)
             if default_roundtrip is None or default != default_roundtrip:
                 raise ValueError(f'{self}: Invalid Default Roundtrip: {default!r} != {default_roundtrip!r}')
         if self.ldefault not in self:
-            raise ValueError(f'{self}: Invalid Default Label: {self.ldefault!r}')
+            dmessage = '; set a valid "default"' if self.ldefault == '0' else ''
+            raise ValueError(f'{self}: Invalid Default Label: {self.ldefault!r}{dmessage}')
         if __debug__:
             if self.fn in (int,):
                 warnings.warn('Bad `fn`, throws exception on error, use `fn.valNumber`', stacklevel=3)
@@ -278,9 +302,10 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
     def __str__(self):
         lrange = LIMIT_ISTRING[self.imin]
         rrange = LIMIT_ISTRING[not self.imax]
-        return '%s%s, %s%s' % (  # noqa: UP031
+        return '%s%s%s, %s%s' % (  # noqa: UP031
             lrange,
             self.min_value,
+            '' if self.step == 1 else f', step={self.step}',
             self.max_value,
             rrange,
         )
@@ -305,7 +330,7 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
         for n in self.rrange:
             yield str(n)
 
-    def __getitem__(self, label: str) -> limT:
+    def __getitem__(self, label: str) -> int:
         '''Get the value corresponding to the label.
 
         If the value is invalid, `KeyError` is raised.
@@ -336,34 +361,37 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
 
         The only parameters that should be changed are:
 
-        - imin: Include minimum value itself.
-        - imax: Include maximum value itself.
+        - imin: Include the limit on the minimum value itself.
+        - imax: Include the limit on the maximum value itself.
+        - step: The step amount for the range of values.
         - default: Default value to be set.
         '''
         if __debug__:
-            extras = set(kwargs) - set(['imin', 'imax', 'default'])
+            extras = set(kwargs) - set(['imin', 'imax', 'step', 'default'])
             if len(extras) > 0:
                 warnings.warn(f'Use a brand new limit: w({", ".join(extras)})', stacklevel=2)
         return self.__class__(**{
-            'min_entry': self.min, 'max_entry': self.max,
+            'min_entry': self.min, 'max_entry': self.max, 'step': self.step,
             'fn': self.fn,
             'imin': self.imin, 'imax': self.imax,
             'default': self.ldefault,
             **kwargs,
         })
 
-    def get_label(self, value: typing.Optional[limT], *, invalid: str = '') -> str:
+    def get_label(self, value: typing.Optional[typing.Any], *, invalid: str = '') -> str:
         '''Get a label that produces the given value.
 
+        The invalid label is produced for all non-`int` values.
+
         Args:
             value: The value object, or `None` to produce an invalid label.
             invalid: The invalid label. Make sure this is really invalid.
         '''
         label = str(value)
-        if value is not None and label in self:
+        if value is not None and isinstance(value, int) and label in self:
             return label
         else:
-            assert invalid not in self, 'Valid "invalid" value: {invalid}'
+            assert invalid not in self, 'Valid "invalid" value: {invalid!r}'
             return invalid
 
     def count_padsize(self, base: int = 10) -> int:
@@ -384,10 +412,10 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
         '''
         assert base > 0
         assert isinstance(self.min, int) and isinstance(self.max, int), f'Unsupported Limit type: {self.min!r}/{self.max!r}'
-        maxn: limT = max(abs(self.min), abs(self.max))
+        maxn: int = max(abs(self.min), abs(self.max))
         return math.ceil(math.log(maxn + 1, base))
 
-    def format_value(self, value: limT, *,
+    def format_value(self, value: int, *,
                      base: typing.Literal[2, 8, 10, 16],
                      invalid: str = '') -> str:
         '''Format a given ``value`` with the strictly necessary amount of
@@ -413,8 +441,8 @@ class LimitBounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
 
 @dataclass
 class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
-    '''Represent an unbounded range to limit a value, generic for any `number
-    <numbers.Number>` type.
+    '''Represent a potentially unbounded range to limit a value, generic for
+    any `number <numbers.Number>` type.
 
     The range can be made to remove the limits themselves with ``imin`` and
     ``imax``. The default is the limit being closed on both sides.
@@ -443,6 +471,10 @@ class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
     Arguments:
         fn: A function that turns a string into `number <numbers.Number>`.
             Usually some kind of parser.
+        step: The step amount for the range of values.
+            Changing this value is unsupported for now, it included for
+            compatibility with `LimitBounded`.
+            Defaults to ``None``.
         imin: Include the limit on the minimum value itself.
             Defaults to `True`.
         imax: Include the limit on the maximum value itself.
@@ -465,12 +497,12 @@ class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
     fn: typing.Callable[[str], typing.Optional[limT]]
     imin: bool = True
     imax: bool = True
+    step: typing.Optional[limT] = None
     # Calculations
     min: typing.Optional[limT] = dc_field(init=False)
     max: typing.Optional[limT] = dc_field(init=False)
     min_value: str = dc_field(init=False)
     max_value: str = dc_field(init=False)
-    # step: typing.Optional[limT] = None  # TODO: Support
     ldefault: str = dc_field(init=False)
     # __init__ variable
     default: InitVar[typing.Union[limT, str, None]] = None
@@ -505,6 +537,8 @@ class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
         else:  # isinstance(min_entry, limT)
             self.max = max_entry
             self.max_value = str(self.max)
+        if self.step is not None:
+            raise ValueError(f'Invalid Step: Not Supported `{self.step!r}`')
         # Default
         if default is None:
             self.ldefault = {
@@ -539,6 +573,7 @@ class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
     def __str__(self):
         lrange = LIMIT_ISTRING[self.imin]
         rrange = LIMIT_ISTRING[not self.imax]
+        # TODO: Support "step"
         return '%s%s, %s%s' % (  # noqa: UP031
             lrange,
             self.min_value,
@@ -588,16 +623,17 @@ class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
 
         The only parameters that should be changed are:
 
-        - imin: Include minimum value itself.
-        - imax: Include maximum value itself.
+        - imin: Include the limit on the minimum value itself.
+        - imax: Include the limit on the maximum value itself.
+        - step: The step amount for the range of values.
         - default: Default value to be set.
         '''
         if __debug__:
-            extras = set(kwargs) - set(['imin', 'imax', 'default'])
+            extras = set(kwargs) - set(['imin', 'imax', 'step', 'default'])
             if len(extras) > 0:
                 warnings.warn(f'Use a brand new limit: w({", ".join(extras)})', stacklevel=2)
         return self.__class__(**{
-            'min_entry': self.min, 'max_entry': self.max,
+            'min_entry': self.min, 'max_entry': self.max, 'step': self.step,
             'fn': self.fn,
             'imin': self.imin, 'imax': self.imax,
             'infinite': self.min is None and self.max is None,
@@ -616,17 +652,5 @@ class LimitUnbounded(typing.Mapping[str, limT], typing.Generic[limT], VSpec):
         if value is not None and label in self:
             return label
         else:
-            assert invalid not in self, 'Valid "invalid" value: {invalid}'
+            assert invalid not in self, 'Valid "invalid" value: {invalid!r}'
             return invalid
-
-    def as_bounded(self) -> 'LimitBounded':
-        '''Create a `LimitBounded` with the same parameters.
-
-        Raises `ValueError` if this is impossible.
-        '''
-        if self.min is None or self.max is None:
-            raise ValueError(f'{self}: Not bounded, impossible to convert')
-        return LimitBounded(self.min, self.max, fn=self.fn,
-                            imin=self.imin, imax=self.imax,
-                            default=self.ldefault,
-                            )
