@@ -5,6 +5,7 @@ import typing
 import tkinter as tk
 from functools import wraps
 import collections.abc
+import warnings
 
 from . import model as tkmilan_model
 from . import validation
@@ -42,7 +43,7 @@ def trace(var: Variable, function: typing.Callable, *,
           **kwargs: typing.Any) -> str:
     '''Trace the variable ``var``.
 
-    There is no Python documentation, see ``Tk`` :tk:`trace variable
+    There is no Python documentation, see ``Tk`` :tcl:`trace variable
     <trace.html#M14>` documentation.
 
     The ``function`` arguments will be:
@@ -265,6 +266,9 @@ class SpecParsed(Spec, typing.Generic[spvT]):
             In debug mode, when setting the variable using a `VState`, the
             ``value`` itself is re-validated.
 
+            In debug mode, mark the variable with `_complexValidation = True`
+            to skip the validation about broken VState.
+
         See Also:
             The base class version is `Spec.set`. It is compatible, but with
             different semantics.
@@ -272,7 +276,10 @@ class SpecParsed(Spec, typing.Generic[spvT]):
         label: str  # Support setting labels and VState
         if isinstance(state, tkmilan_model.VState):
             label = state.label
-            assert state.value == self.container.get(label), f'{self} has a weird value'
+            if __debug__:
+                wvalue = self.container.get(label)
+                if state.value != wvalue and getattr(self, '_complexValidation', False) is False:
+                    warnings.warn(f'{self}: Setting broken VState: new={state!r} value={self.container.get(label)!r}')
         else:  # Setting a simple label
             assert isinstance(state, str)
             label = state
@@ -396,6 +403,11 @@ class StaticList(SpecCountable):
         spec: The list validation object.
         name: Name of the variable (passed to `String`).
             Optional, defaults to an autogenerated name.
+
+    .. note::
+
+        The ``name`` is defined in a global namespace, common to the
+        entire application.
     '''
     def __init__(self, spec: validation.StaticList, *,
                  name: typing.Optional[str] = None,
@@ -416,6 +428,11 @@ class StaticMap(SpecCountable, SpecParsed[spvT]):
         spec: The mapping validation object.
         name: Name of the variable (passed to `String`).
             Optional, defaults to an autogenerated name.
+
+    .. note::
+
+        The ``name`` is defined in a global namespace, common to the
+        entire application.
     '''
     def __init__(self, spec: validation.StaticMap[spvT], *,
                  name: typing.Optional[str] = None,
@@ -484,9 +501,14 @@ class Limit(SpecParsed[int]):
             # Sub-Class, instance as usual
             return super().__new__(cls)
 
-    def get_spinargs(self) -> typing.Tuple[float, float, float]:
+    def get_spinargs(self, *, dincrement: float = 1.0) -> typing.Tuple[float, float, float]:
         '''Get all `SpinboxN` value arguments.
 
+        Arguments:
+            dincrement: Default ``increment`` value, when the limit does not
+                support it.
+                Defaults to ``1.0``.
+
         Return Value:
             Returns a tuple with three floating point values:
 
@@ -495,7 +517,6 @@ class Limit(SpecParsed[int]):
             - ``increment``
         '''
         infinite: float = float('+inf')
-        sincrement = 1.0  # TODO: Support "step"
         sfrom: float
         if self.container.min is None:
             sfrom = -1 * infinite
@@ -508,10 +529,17 @@ class Limit(SpecParsed[int]):
         else:
             assert isinstance(self.container.max, typing.SupportsFloat), f'Invalid max value: {self.container.max!r}'
             sto = float(self.container.max)
+        sincrement: float = float(self.container.step or dincrement)
         if not self.container.imin:
             sfrom += sincrement
         if not self.container.imax:
             sto -= sincrement
+        if __debug__:
+            if isinstance(self.container, validation.LimitBounded):
+                rrange = self.container.rrange
+                assert int(sfrom) in rrange
+                assert int(sto) in rrange
+                assert int(sincrement) == int(rrange.step)
         return sfrom, sto, sincrement
 
     def getLabel(self, value: typing.Optional[spvT]) -> str:
@@ -534,6 +562,11 @@ class LimitBounded(Limit, SpecCountable):
         limit: The `LimitBounded <validation.LimitBounded>` range object..
         name: Name of the variable (passed to `String`).
             Optional, defaults to an autogenerated name.
+
+    .. note::
+
+        The ``name`` is defined in a global namespace, common to the
+        entire application.
     '''
     def __init__(self, limit: validation.LimitBounded, *,
                  name: typing.Optional[str] = None,
@@ -556,6 +589,11 @@ class LimitUnbounded(Limit):
         limit: The `LimitUnbounded <validation.LimitUnbounded>` range object.
         name: Name of the variable (passed to `String`).
             Optional, defaults to an autogenerated name.
+
+    .. note::
+
+        The ``name`` is defined in a global namespace, common to the
+        entire application.
     '''
     def __init__(self, limit: validation.LimitUnbounded, *,
                  name: typing.Optional[str] = None,
@@ -581,3 +619,55 @@ if __debug__:
     map_subclasses = set(MAP_VALIDATIONS.keys())
     delta_subclasses_names = [cls.__qualname__ for cls in map_subclasses.symmetric_difference(vspec_subclasses)]
     assert len(delta_subclasses_names) == 0, f'Missing `MAP_VALIDATIONS` for {" ".join(delta_subclasses_names)}'
+
+
+# Custom Validation Settings
+def __vfn_limit(vstate, why):
+    assert why is not None, 'Need `fnSimple=False`'
+    wvalidation = why.widget
+    assert isinstance(wvalidation.variable, Spec), f'{wvalidation}: Not a widget with Spec variable'
+    assert why.tt is not None, f'{wvalidation}: Not Tooltip configured'  # Don't repeat the tooltip handling
+    vspec = wvalidation.variable.container
+    vstate_raw = vspec.fn(vstate.label)
+    if vstate.valid:
+        # Valid; No Tooltip
+        message = ''
+        why.tt.disable()
+    else:
+        # Invalid:
+        if vstate_raw is None:
+            # Invalid: bad Spec
+            message = 'Spec Violation'
+        else:
+            # Invalid: outside the value spec
+            message = f'Value out of Spec: {vspec}'
+        why.tt.enable()
+    if __debug__:
+        from . import TooltipValidation  # Avoid circular import
+        assert isinstance(why.tt, TooltipValidation)
+    why.tt.wstate = message
+    return vstate.valid
+
+
+ValidationLimit = tkmilan_model.VSettings(
+    fnSimple=False, ttSimple=False,
+    fn=__vfn_limit,
+)
+'''Configure widget validation to enhance the tooltip information with `Spec`
+violation information.
+
+This distinguishes between an invalid value, and a value outside the `Spec`.
+This is more apparent on `Limit`.
+
+This is only experimental code, but should work as intended on this narrow
+case.
+
+To be used as:
+
+.. code:: python
+
+    Widget(self, ...
+                validation=var.ValidationLimit,
+                ...
+                ).putTooltip()
+'''
