@@ -4,7 +4,7 @@ import logging
 
 from defaultlist import defaultlist
 
-from tkmilan.model import Direction, VState
+from tkmilan.model import Direction, VState, NotebookTabOrder
 
 logger = logging.getLogger(__name__)
 
@@ -187,6 +187,31 @@ class Test_VState(unittest.TestCase):
         self.assertNotEqual(i1_1, i2_2)
 
 
+class Test_NotebookTabOrder(unittest.TestCase):
+    def test_creation_clean(self):
+        assert __debug__
+        nto_empty = NotebookTabOrder(())
+        self.assertEqual(len(nto_empty.shown), 0)
+        self.assertEqual(len(nto_empty.disabled), 0)
+        nto_basic = NotebookTabOrder(('1', '2', '3'), disabled=set(('2')))
+        self.assertEqual(len(nto_basic.shown), 3)
+        self.assertEqual(len(nto_basic.disabled), 1)
+        self.assertTupleEqual(nto_basic.active, ('1', '3'))
+        nto_lists = NotebookTabOrder(['1', '2', '3'], disabled=['3'])
+        self.assertIsInstance(nto_lists.shown, typing.Tuple)
+        self.assertIsInstance(nto_lists.disabled, typing.Set)
+        self.assertTupleEqual(nto_lists.active, ('1', '2'))
+
+    def test_creation_error(self):
+        assert __debug__
+        # Repeated `shown`
+        with self.assertRaises(AssertionError):
+            NotebookTabOrder(['1', '2', '1'])
+        # Bad disabled keys
+        with self.assertRaises(AssertionError):
+            NotebookTabOrder(('1', '2', '3'), disabled=set('a'))
+
+
 if __name__ == '__main__':
     import sys
     logs_lvl = logging.DEBUG if '-v' in sys.argv else logging.INFO
